# Copyright 2015 NEC Corporation
# All Rights Reserved.
#
# Copyright (c) 2022 Dell Inc. or its subsidiaries.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

from tempest import config  # noqa


# NOTE(TheJulia): The following options are loaded into a tempest
# plugin configuration option via plugin.py.
ironic_service_option = cfg.BoolOpt('ironic',
                                    default=False,
                                    help='Whether or not ironic is expected '
                                    'to be available')

inspector_service_option = cfg.BoolOpt("ironic_inspector",
                                       default=False,
                                       help="Whether or not ironic-inspector "
                                       "is expected to be available")

ironic_scope_enforcement = cfg.BoolOpt('ironic',
                                       default=True,
                                       help='Wheter or not ironic is '
                                            'exepcted to enforce auth '
                                            'scope.')

inspector_scope_enforcement = cfg.BoolOpt('ironic_inspector',
                                          default=True,
                                          help='Whether or not '
                                               'ironic-inspector is expected '
                                               'to enforce auth scope.')

baremetal_group = cfg.OptGroup(name='baremetal',
                               title='Baremetal provisioning service options',
                               help='When enabling baremetal tests, Nova '
                                    'must be configured to use the Ironic '
                                    'driver. The following parameters for the '
                                    '[compute] section must be disabled: '
                                    'console_output, interface_attach, '
                                    'live_migration, pause, rescue, resize, '
                                    'shelve, snapshot, and suspend')

# The bulk of the embedded configuration is below.

baremetal_introspection_group = cfg.OptGroup(
    name="baremetal_introspection",
    title="Baremetal introspection service options",
    help="When enabling baremetal introspection tests,"
         "Ironic must be configured.")

baremetal_features_group = cfg.OptGroup(
    name='baremetal_feature_enabled',
    title="Enabled Baremetal Service Features")

BaremetalGroup = [
    cfg.StrOpt('catalog_type',
               default='baremetal',
               help="Catalog type of the baremetal provisioning service"),
    cfg.StrOpt('driver',
               default='fake-hardware',
               help="Driver name to use for API tests"),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the baremetal provisioning"
                    " service"),
    cfg.StrOpt('root_device_name',
               default='/dev/md0',
               help="Root device name to be used for node deployment"),
    cfg.IntOpt('deploywait_timeout',
               default=15,
               help="Timeout for Ironic node to reach the "
                    "wait-callback state after powering on."),
    cfg.IntOpt('active_timeout',
               default=300,
               help="Timeout for Ironic node to completely provision"),
    cfg.IntOpt('anaconda_active_timeout',
               default=3600,
               help="Timeout for Ironic node to completely provision "
                    "when using the anaconda deployment interface."),
    cfg.IntOpt('association_timeout',
               default=30,
               help="Timeout for association of Nova instance and Ironic "
                    "node"),
    cfg.IntOpt('inspect_timeout',
               default=300,
               help="Timeout for inspecting an Ironic node."),
    cfg.IntOpt('power_timeout',
               default=60,
               help="Timeout for Ironic power transitions."),
    cfg.IntOpt('unprovision_timeout',
               default=300,
               help="Timeout for unprovisioning an Ironic node. "
                    "Takes longer since Kilo as Ironic performs an extra "
                    "step in Node cleaning."),
    cfg.IntOpt('rescue_timeout',
               default=300,
               help="Timeout for rescuing an Ironic node."),
    cfg.IntOpt('unrescue_timeout',
               default=300,
               help="Timeout for unrescuing an Ironic node."),
    cfg.StrOpt('min_microversion',
               help="Lower version of the test target microversion range. "
                    "The format is 'X.Y', where 'X' and 'Y' are int values. "
                    "Tempest selects tests based on the range between "
                    "min_microversion and max_microversion. "
                    "If both values are None, Tempest avoids tests which "
                    "require a microversion."),
    cfg.StrOpt('max_microversion',
               default='latest',
               help="Upper version of the test target microversion range. "
                    "The format is 'X.Y', where 'X' and 'Y' are int values. "
                    "Tempest selects tests based on the range between "
                    "min_microversion and max_microversion. "
                    "If both values are None, Tempest avoids tests which "
                    "require a microversion."),
    cfg.BoolOpt('use_provision_network',
                default=False,
                help="Whether the Ironic/Neutron tenant isolation is enabled"),
    cfg.StrOpt('whole_disk_image_ref',
               help="UUID of the wholedisk image to use in the tests."),
    cfg.StrOpt('whole_disk_image_url',
               help="An http link to the wholedisk image to use in the "
                    "tests."),
    cfg.StrOpt('whole_disk_image_checksum',
               help="An checksum of the image. Recommend SHA256 or SHA512"
                    "as MD5 is deprecated."),
    cfg.StrOpt('partition_image_ref',
               help="UUID of the partitioned image to use in the tests."),
    cfg.StrOpt('ramdisk_iso_image_ref',
               help=("UUID (or url) of an ISO image for the ramdisk boot "
                     "tests.")),
    cfg.StrOpt('storage_inventory_file',
               help="Path to storage inventory file for RAID cleaning tests."),
    cfg.StrOpt('export_location',
               help="Export config location for configuration molds."),
    cfg.StrOpt('import_location',
               help="Import config location for configuration molds."),
    cfg.StrOpt('rollback_import_location',
               help="Rollback import config location for configuration "
                    "molds. Optional. If not provided, rollback is skipped."),
    # TODO(TheJulia): For now, anaconda can be url based and we can move in
    # to being tested with glance as soon as we get a public stage2 image.
    cfg.StrOpt('anaconda_image_ref',
               help="URL of an anaconda repository to set as image_source"),
    cfg.StrOpt('anaconda_kernel_ref',
               help="URL of the kernel to utilize for anaconda deploys."),
    cfg.StrOpt('anaconda_initial_ramdisk_ref',
               help="URL of the initial ramdisk to utilize for anaconda "
                    "deploy operations."),
    cfg.StrOpt('anaconda_stage2_ramdisk_ref',
               help="URL of the anaconda second stage ramdisk. Presence of "
                    "this setting will also determine if a stage2 specific "
                    "anaconda test is run, or not."),
    cfg.StrOpt('anaconda_exit_test_at',
               default='heartbeat',
               choices=['heartbeat', 'active'],
               help='When to end the anaconda test job at. Due to '
                    'the use model of the anaconda driver, as well '
                    'as the performance profile, the anaconda test is '
                    'normally only executed until we observe a heartbeat '
                    'operation indicating that anaconda *has* booted and '
                    'successfully parsed the URL.'),
    cfg.ListOpt('enabled_drivers',
                default=['fake', 'pxe_ipmitool', 'agent_ipmitool'],
                help="List of Ironic enabled drivers."),
    cfg.ListOpt('enabled_hardware_types',
                default=['ipmi'],
                help="List of Ironic enabled hardware types."),
    cfg.ListOpt('enabled_bios_interfaces',
                default=['fake'],
                help="List of Ironic enabled bios interfaces."),
    cfg.ListOpt('enabled_deploy_interfaces',
                default=['iscsi', 'direct'],
                help="List of Ironic enabled deploy interfaces."),
    cfg.ListOpt('enabled_rescue_interfaces',
                default=['no-rescue'],
                help="List of Ironic enabled rescue interfaces."),
    cfg.ListOpt('enabled_boot_interfaces',
                default=['fake', 'pxe'],
                help="List of Ironic enabled boot interfaces."),
    cfg.ListOpt('enabled_raid_interfaces',
                default=['no-raid', 'agent'],
                help="List of Ironic enabled RAID interfaces."),
    cfg.ListOpt('enabled_management_interfaces',
                default=['fake', 'ipmitool', 'noop'],
                help="List of Ironic enabled management interfaces."),
    cfg.ListOpt('enabled_power_interfaces',
                default=['fake', 'ipmitool'],
                help="List of Ironic enabled power interfaces."),
    cfg.ListOpt('enabled_inspect_interfaces',
                default=['no-inspect'],
                help="List of Ironic enabled inspect interfaces."),
    cfg.StrOpt('default_rescue_interface',
               help="Ironic default rescue interface."),
    cfg.StrOpt('firmware_image_url',
               help="Image URL of firmware image file supported by "
                    "update_firmware clean step."),
    cfg.StrOpt('firmware_image_checksum',
               help="SHA1 checksum of firmware image file."),
    cfg.StrOpt('firmware_rollback_image_url',
               help="Image URL of firmware update cleaning step's "
                    "rollback image. Optional. If not provided, "
                    "rollback is skipped."),
    cfg.StrOpt('firmware_rollback_image_checksum',
               help="SHA1 checksum of firmware rollback image file. "
                    "This is required if firmware_rollback_image_url is set."),
    cfg.IntOpt('adjusted_root_disk_size_gb',
               min=0,
               help="Ironic adjusted disk size to use in the standalone tests "
                    "as instance_info/root_gb value."),
    cfg.BoolOpt('partition_netboot',
                default=True,
                help="Treat partition images as netbooted as opposed to "
                     "attempting to populate a boot loader. IF cirros is "
                     "being used, this option should be set to True as "
                     "it lacks the needed components to make it locally "
                     "from a partition image."),
    cfg.StrOpt('boot_mode',
               default='bios',
               choices=['bios', 'uefi'],
               help="The desired boot_mode to be used on testing nodes."),
    cfg.StrOpt('default_boot_option',
               # No good default here, we need to actually set it.
               help="The default boot option the testing nodes are using."),
    cfg.BoolOpt("rebuild_remote_dhcpless",
                default=True,
                help="If we should issue a rebuild request when testing "
                     "dhcpless virtual media deployments. This may be useful "
                     "if bug 2032377 is not fixed in the agent ramdisk. This "
                     "test is functionally incompatible with Temporary URLs "
                     "as they may timeout while the test is running."),
    cfg.StrOpt("public_subnet_id",
               help="The public subnet ID where routers will be bound for "
                    "testing purposes with the dhcp-less test scenario."),
    cfg.StrOpt("public_subnet_ip",
               help="The public subnet IP to bind the public router to for "
                    "dhcp-less testing.")
]

BaremetalFeaturesGroup = [
    cfg.BoolOpt('ipxe_enabled',
                default=True,
                help="Defines if IPXE is enabled"),
    cfg.BoolOpt('adoption',
                # Defaults to False since it's a destructive operation AND it
                # requires the plugin to be able to read ipmi_password.
                default=False,
                help="Defines if adoption is enabled"),
    cfg.BoolOpt('software_raid',
                default=False,
                help="Defines if software RAID is enabled (available "
                     "starting with Train). Requires at least two disks "
                     "on testing nodes."),
    cfg.BoolOpt('deploy_time_raid',
                default=False,
                help="Defines if in-band RAID can be built in deploy time "
                     "(possible starting with Victoria)."),
    cfg.BoolOpt('dhcpless_vmedia',
                default=False,
                help="Defines if it is possible to execute DHCP-Less "
                     "deployment of baremetal nodes through virtual media. "
                     "This test requires full OS images with configuration "
                     "support for embedded network metadata through glean "
                     "or cloud-init, and thus cannot be executed with "
                     "most default job configurations."),
    cfg.BoolOpt('trunks_supported',
                default=False,
                help="Define if trunks are supported by networking driver "
                     "with baremetal nodes."),
]

BaremetalIntrospectionGroup = [
    cfg.StrOpt('catalog_type',
               default='baremetal-introspection',
               help="Catalog type of the baremetal provisioning service"),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the baremetal introspection"
                    " service"),
    cfg.IntOpt('introspection_sleep',
               default=30,
               help="Introspection sleep before check status"),
    cfg.IntOpt('introspection_timeout',
               default=600,
               help="Introspection time out"),
    cfg.IntOpt('introspection_start_timeout',
               default=90,
               help="Timeout to start introspection"),
    cfg.IntOpt('hypervisor_update_sleep',
               default=60,
               help="Time to wait until nova becomes aware of "
                    "bare metal instances"),
    cfg.IntOpt('hypervisor_update_timeout',
               default=300,
               help="Time out for wait until nova becomes aware of "
                    "bare metal instances"),
    # NOTE(aarefiev): status_check_period default is 60s, but checking
    # node state takes some time(API call), so races appear here,
    # 80s would be enough to make one more check.
    cfg.IntOpt('ironic_sync_timeout',
               default=80,
               help="Time it might take for Ironic--Inspector "
                    "sync to happen"),
    cfg.IntOpt('discovery_timeout',
               default=300,
               help="Time to wait until new node would enrolled in "
                    "ironic"),
    cfg.BoolOpt('auto_discovery_feature',
                default=False,
                help="Is the auto-discovery feature enabled. Enroll hook "
                     "should be specified in node_not_found_hook - processing "
                     "section of inspector.conf"),
    cfg.StrOpt('auto_discovery_default_driver',
               # TODO(dtantsur): change to fake-hardware when Queens is no
               # longer supported.
               default='fake',
               help="The driver expected to be set on newly discovered nodes. "
                    "Only has effect with auto_discovery_feature is True."),
    cfg.StrOpt('auto_discovery_target_driver',
               help="The driver to set on the newly discovered nodes. "
                    "Only has effect with auto_discovery_feature is True."),
    cfg.StrOpt('data_store',
               help="The storage backend for storing introspection data."),
]



#
# Copyright 2019 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from ironic_tempest_plugin import manager
from ironic_tempest_plugin.tests.scenario import baremetal_manager

CONF = config.CONF


class BaremetalSingleTenant(baremetal_manager.BaremetalScenarioTest,
                            manager.NetworkScenarioTest):
    """Check "No L2 isolation" of baremetal and VM instances of same tenant:

    * Create a keypair, network, subnet and router for the primary tenant
    * Boot 2 instances in the same tenant's network using the keypair
    * Associate floating ips to both instances
    * Verify there is L3 connectivity between instances of same tenant
    * Verify connectivity between instances floating IP's
    * Delete both instances
    """

    credentials = ['primary', 'alt', 'admin', 'system_admin']

    @classmethod
    def skip_checks(cls):
        super(BaremetalSingleTenant, cls).skip_checks()
        if not CONF.baremetal.use_provision_network:
            msg = 'Ironic/Neutron tenant isolation is not configured.'
            raise cls.skipException(msg)

    def create_tenant_network(self, clients, tenant_cidr):
        network = self.create_network(
            networks_client=clients.networks_client,
            project_id=clients.credentials.project_id)
        router = self.get_router(
            client=clients.routers_client,
            project_id=clients.credentials.tenant_id)

        extra_subnet_args = {}
        if CONF.validation.ip_version_for_ssh == 6:
            extra_subnet_args['ipv6_address_mode'] = 'dhcpv6-stateless'
            extra_subnet_args['ipv6_ra_mode'] = 'dhcpv6-stateless'
            extra_subnet_args['gateway_ip'] = 'fd00:33::1'

        result = clients.subnets_client.create_subnet(
            name=data_utils.rand_name('subnet'),
            network_id=network['id'],
            tenant_id=clients.credentials.tenant_id,
            ip_version=CONF.validation.ip_version_for_ssh,
            cidr=tenant_cidr, **extra_subnet_args)
        subnet = result['subnet']
        clients.routers_client.add_router_interface(router['id'],
                                                    subnet_id=subnet['id'])
        self.addCleanup(clients.subnets_client.delete_subnet, subnet['id'])
        self.addCleanup(clients.routers_client.remove_router_interface,
                        router['id'], subnet_id=subnet['id'])

        return network, subnet, router

    def verify_l3_connectivity(self, source_ip, private_key,
                               destination_ip, conn_expected=True):
        remote = self.get_remote_client(source_ip, private_key=private_key)
        remote.validate_authentication()

        cmd = 'ping %s -c4 -w4 || exit 0' % destination_ip
        success_substring = " bytes from %s" % destination_ip
        output = remote.exec_command(cmd)
        if conn_expected:
            self.assertIn(success_substring, output)
        else:
            self.assertNotIn(success_substring, output)

    def tenancy_check(self, use_vm=False):

        ip_version = CONF.validation.ip_version_for_ssh

        tenant_cidr = '10.0.100.0/24'
        if ip_version == 6:
            tenant_cidr = 'fd00:33::/64'

        keypair = self.create_keypair()
        network, subnet, router = self.create_tenant_network(
            self.os_primary, tenant_cidr)

        instance1, node1 = self.boot_instance(
            clients=self.os_primary,
            keypair=keypair,
            net_id=network['id'],
        )

        fixed_ip1 = instance1['addresses'][network['name']][0]['addr']
        if ip_version == 6:
            floating_ip1 = fixed_ip1
        else:
            floating_ip1 = self.create_floating_ip(
                instance1,
            )['floating_ip_address']
        self.check_vm_connectivity(ip_address=floating_ip1,
                                   private_key=keypair['private_key'],
                                   server=instance1)

        if use_vm:
            # Create VM on compute node
            instance2 = self.create_server(
                clients=self.os_primary,
                key_name=keypair['name'],
                flavor=CONF.compute.flavor_ref,
                networks=[{'uuid': network['id']}]
            )
        else:
            # Create BM
            instance2, node2 = self.boot_instance(
                keypair=keypair,
                clients=self.os_primary,
                net_id=network['id'],
            )
        fixed_ip2 = \
            instance2['addresses'][network['name']][0]['addr']
        if ip_version == 6:
            floating_ip2 = fixed_ip2
        else:
            floating_ip2 = self.create_floating_ip(
                instance2,
                client=self.os_primary.floating_ips_client
            )['floating_ip_address']
        self.check_vm_connectivity(
            ip_address=floating_ip2,
            private_key=keypair['private_key'],
            server=instance2)

        self.verify_l3_connectivity(
            floating_ip2,
            keypair['private_key'],
            fixed_ip1,
            conn_expected=True
        )
        self.verify_l3_connectivity(
            floating_ip1,
            keypair['private_key'],
            fixed_ip2,
            conn_expected=True
        )
        self.verify_l3_connectivity(
            floating_ip1,
            keypair['private_key'],
            floating_ip2,
            conn_expected=True
        )
        self.terminate_instance(
            instance=instance2,
            servers_client=self.os_primary.servers_client)
        self.terminate_instance(instance=instance1)

    @decorators.idempotent_id('8fe15552-3788-11e9-b599-74e5f9e2a801')
    @utils.services('compute', 'image', 'network')
    def test_baremetal_single_tenant(self):
        if CONF.service_available.nova:
            self.skipTest('Compute service Nova is running,'
                          ' BM to BM test will be skipped,'
                          ' to save test execution time')
        self.tenancy_check()

    @decorators.idempotent_id('90b3b6be-3788-11e9-b599-74e5f9e2a801')
    @utils.services('compute', 'image', 'network')
    def test_baremetal_vm_single_tenant(self):
        if not CONF.service_available.nova:
            self.skipTest('Compute service Nova is disabled,'
                          ' VM is required to run this test')
        self.tenancy_check(use_vm=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from ironic_tempest_plugin.common import waiters
from ironic_tempest_plugin.tests.api import base

CONF = config.CONF


class Base(base.BaseBaremetalTest):
    @classmethod
    def provide_and_power_off_node(cls, node_id, cleaning_timeout=None):
        cls.provide_node(node_id, cleaning_timeout)
        # Force non-empty power state, otherwise allocation API won't pick it
        cls.client.set_node_power_state(node_id, 'power off')
        waiters.wait_for_bm_node_status(cls.client, node_id,
                                        'power_state', 'power off')

    def setUp(self):
        super(Base, self).setUp()

        # Generate a resource class to prevent parallel tests from clashing
        # with each other.
        self.resource_class = uuidutils.generate_uuid()

        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(
            self.chassis['uuid'],
            resource_class=self.resource_class,
            # Fake deploy interface to avoid stop  when automated
            # cleaning is on.
            deploy_interface='fake',
            # noop network interface in case a cleaning/provisioning network
            # is not defined.
            network_interface='noop')
        self.provide_and_power_off_node(self.node['uuid'])


class TestAllocations(Base):
    """Tests for baremetal allocations."""

    min_microversion = '1.52'

    @decorators.idempotent_id('9203ea28-3c61-4108-8498-22247b654ff6')
    def test_create_show_allocation(self):
        self.assertIsNone(self.node['allocation_uuid'])
        _, body = self.create_allocation(self.resource_class)
        uuid = body['uuid']

        self.assertTrue(uuid)
        self.assertEqual('allocating', body['state'])
        self.assertEqual(self.resource_class, body['resource_class'])
        self.assertIsNone(body['last_error'])
        self.assertIsNone(body['node_uuid'])

        _, body = waiters.wait_for_allocation(self.client, uuid)
        self.assertEqual('active', body['state'])
        self.assertEqual(self.resource_class, body['resource_class'])
        self.assertIsNone(body['last_error'])
        self.assertEqual(self.node['uuid'], body['node_uuid'])

        _, body2 = self.client.show_node_allocation(body['node_uuid'])
        self.assertEqual(body, body2)

        _, node = self.client.show_node(self.node['uuid'])
        self.assertEqual(uuid, node['allocation_uuid'])

    @decorators.idempotent_id('eb074d06-e5f4-4fb4-b992-c9929db488ae')
    def test_create_allocation_with_traits(self):
        _, node2 = self.create_node(self.chassis['uuid'],
                                    resource_class=self.resource_class,
                                    deploy_interface='fake')
        self.client.set_node_traits(node2['uuid'], ['CUSTOM_MEOW'])
        self.provide_and_power_off_node(node2['uuid'])

        _, body = self.create_allocation(self.resource_class,
                                         traits=['CUSTOM_MEOW'])
        uuid = body['uuid']

        self.assertTrue(uuid)
        self.assertEqual('allocating', body['state'])
        self.assertEqual(['CUSTOM_MEOW'], body['traits'])
        self.assertIsNone(body['last_error'])

        _, body = waiters.wait_for_allocation(self.client, uuid)
        self.assertEqual('active', body['state'])
        self.assertEqual(['CUSTOM_MEOW'], body['traits'])
        self.assertIsNone(body['last_error'])
        self.assertEqual(node2['uuid'], body['node_uuid'])

    @decorators.idempotent_id('12d19297-f35a-408a-8b1e-3cd244e30abe')
    def test_create_allocation_candidate_node(self):
        node_name = 'allocation-test-1'
        _, node2 = self.create_node(self.chassis['uuid'],
                                    resource_class=self.resource_class,
                                    name=node_name,
                                    deploy_interface='fake',
                                    network_interface='noop')
        self.provide_and_power_off_node(node2['uuid'])

        _, body = self.create_allocation(self.resource_class,
                                         candidate_nodes=[node_name])
        uuid = body['uuid']

        self.assertTrue(uuid)
        self.assertEqual('allocating', body['state'])
        self.assertEqual([node2['uuid']], body['candidate_nodes'])
        self.assertIsNone(body['last_error'])

        _, body = waiters.wait_for_allocation(self.client, uuid)
        self.assertEqual('active', body['state'])
        self.assertEqual([node2['uuid']], body['candidate_nodes'])
        self.assertIsNone(body['last_error'])
        self.assertEqual(node2['uuid'], body['node_uuid'])

    @decorators.idempotent_id('84eb3c21-4e16-4f33-9551-dce0f8689462')
    def test_delete_allocation(self):
        _, body = self.create_allocation(self.resource_class)
        self.client.delete_allocation(body['uuid'])
        self.assertRaises(lib_exc.NotFound, self.client.show_allocation,
                          body['uuid'])

    @decorators.idempotent_id('5e30452d-ee92-4342-82c1-5eea5e55c937')
    def test_delete_allocation_by_name(self):
        name = 'alloc-%s' % uuidutils.generate_uuid()
        _, body = self.create_allocation(self.resource_class, name=name)
        self.client.delete_allocation(name)
        self.assertRaises(lib_exc.NotFound, self.client.show_allocation, name)

    @decorators.idempotent_id('fbbc13bc-86da-438b-af01-d1bc1bab57d6')
    def test_show_by_name(self):
        name = 'alloc-%s' % uuidutils.generate_uuid()
        _, body = self.create_allocation(self.resource_class, name=name)
        _, loaded_body = self.client.show_allocation(name)
        # The allocation will likely have been processed by this time, so do
        # not compare the whole body.
        for field in ('name', 'uuid', 'resource_class'):
            self.assertEqual(body[field], loaded_body[field])

    @decorators.idempotent_id('4ca123c4-160d-4d8d-a3f7-15feda812263')
    def test_list_allocations(self):
        _, body = self.create_allocation(self.resource_class)

        _, listing = self.client.list_allocations()
        self.assertIn(body['uuid'],
                      [i['uuid'] for i in listing['allocations']])

        _, listing = self.client.list_allocations(
            resource_class=self.resource_class)
        self.assertEqual([body['uuid']],
                         [i['uuid'] for i in listing['allocations']])

    @decorators.idempotent_id('092b7148-9ff0-4107-be57-2cfcd21eb5d7')
    def test_list_allocations_by_state(self):
        _, body = self.create_allocation(self.resource_class)
        _, body2 = self.create_allocation(self.resource_class + 'foo2')

        waiters.wait_for_allocation(self.client, body['uuid'])
        waiters.wait_for_allocation(self.client, body2['uuid'],
                                    expect_error=True)

        _, listing = self.client.list_allocations(state='active')
        uuids = [i['uuid'] for i in listing['allocations']]
        self.assertIn(body['uuid'], uuids)
        self.assertNotIn(body2['uuid'], uuids)

        _, listing = self.client.list_allocations(state='error')
        uuids = [i['uuid'] for i in listing['allocations']]
        self.assertNotIn(body['uuid'], uuids)
        self.assertIn(body2['uuid'], uuids)

        _, listing = self.client.list_allocations(state='allocating')
        uuids = [i['uuid'] for i in listing['allocations']]
        self.assertNotIn(body['uuid'], uuids)
        self.assertNotIn(body2['uuid'], uuids)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('bf7e1375-019a-466a-a294-9c1052827ada')
    def test_create_allocation_resource_class_mismatch(self):
        _, body = self.create_allocation(self.resource_class + 'foo')

        _, body = waiters.wait_for_allocation(self.client, body['uuid'],
                                              expect_error=True)
        self.assertEqual('error', body['state'])
        self.assertTrue(body['last_error'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b4eeddee-ca34-44f9-908b-490b78b18486')
    def test_create_allocation_traits_mismatch(self):
        _, body = self.create_allocation(
            self.resource_class, traits=['CUSTOM_DOES_NOT_EXIST'])

        _, body = waiters.wait_for_allocation(self.client, body['uuid'],
                                              expect_error=True)
        self.assertEqual('error', body['state'])
        self.assertTrue(body['last_error'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('2378727f-77c3-4289-9562-bd2f3b147a60')
    def test_create_allocation_node_mismatch(self):
        _, node2 = self.create_node(self.chassis['uuid'],
                                    resource_class=self.resource_class + 'alt',
                                    deploy_interface='fake')
        # Mismatch between the resource class and the candidate node
        _, body = self.create_allocation(
            self.resource_class, candidate_nodes=[node2['uuid']])

        _, body = waiters.wait_for_allocation(self.client, body['uuid'],
                                              expect_error=True)
        self.assertEqual('error', body['state'])
        self.assertTrue(body['last_error'])


class TestAllocationsWithJsonExtSupport(Base):
    """Tests for baremetal allocations to validate appending .json extension

     to an allocation's `name` or `uuid` in API versions 1.90 and prior works.
     """

    max_microversion = '1.90'  # Max API version allowed for testing: 1.90
    min_microversion = '1.90'  # Min API version allowed for testing: 1.90

    @decorators.idempotent_id('d111b85b-a169-440e-9dcd-9b8b6ad8c917')
    def test_create_show_allocation_with_json(self):
        """Show an allocation while appending .json extension to its uuid"""
        self.assertIsNone(self.node['allocation_uuid'])
        _, body = self.create_allocation(self.resource_class)
        uuid = body['uuid']

        self.assertTrue(uuid)
        self.assertEqual('allocating', body['state'])
        self.assertEqual(self.resource_class, body['resource_class'])
        self.assertIsNone(body['last_error'])
        self.assertIsNone(body['node_uuid'])

        _, body = waiters.wait_for_allocation(self.client, uuid)
        self.assertEqual('active', body['state'])
        self.assertEqual(self.resource_class, body['resource_class'])
        self.assertIsNone(body['last_error'])
        self.assertEqual(self.node['uuid'], body['node_uuid'])

        _, body2 = self.client.show_node_allocation(body['node_uuid'])
        self.assertEqual(body, body2)

        _, node = self.client.show_node('%s.json' % self.node['uuid'])
        self.assertEqual(uuid, node['allocation_uuid'])

    @decorators.idempotent_id('b94643f4-f789-4c04-a806-bf769e0bc181')
    def test_delete_allocation_with_json(self):
        """Delete an allocation while appending .json extension to its uuid"""
        _, body = self.create_allocation(self.resource_class)

        self.client.delete_allocation('%s.json' % body['uuid'])

        self.assertRaises(lib_exc.NotFound, self.client.show_allocation,
                          '%s.json' % body['uuid'])


class TestAllocationsWithoutJsonExtSupport(Base):
    """Tests for baremetal allocations to validate appending .json extension

    to an allocation's `name` or `uuid` in API versions later than 1.90 is

    disabled.
    """

    min_microversion = '1.91'  # Min API version allowed for testing: 1.91

    @decorators.idempotent_id('6b851ef4-e364-4a3e-af1d-e6b73c1adec6')
    def test_create_show_allocation_with_json(self):
        """Trying to show an allocation while appending .json ext 404s"""
        self.assertIsNone(self.node['allocation_uuid'])
        _, body = self.create_allocation(self.resource_class)
        uuid = body['uuid']

        self.assertTrue(uuid)
        self.assertEqual('allocating', body['state'])
        self.assertEqual(self.resource_class, body['resource_class'])
        self.assertIsNone(body['last_error'])
        self.assertIsNone(body['node_uuid'])

        _, body = waiters.wait_for_allocation(self.client, uuid)
        self.assertEqual('active', body['state'])
        self.assertEqual(self.resource_class, body['resource_class'])
        self.assertIsNone(body['last_error'])
        self.assertEqual(self.node['uuid'], body['node_uuid'])

        _, body2 = self.client.show_node_allocation(body['node_uuid'])
        self.assertEqual(body, body2)

        self.assertRaises(lib_exc.NotFound, self.client.show_allocation,
                          '%s.json' % body['uuid'])

    @decorators.idempotent_id('cdf36c46-3da7-4306-b265-bcea3ab3bc3f')
    def test_delete_allocation_with_json(self):
        """Trying to delete an allocation while appending .json ext 404s"""
        _, body = self.create_allocation(self.resource_class)
        self.assertRaises(lib_exc.NotFound, self.client.delete_allocation,
                          '%s.json' % body['uuid'])


class TestBackfill(Base):
    """Tests for backfilling baremetal allocations."""

    min_microversion = '1.58'

    @decorators.idempotent_id('10774c1d-6b79-453a-8e26-9bf04ab580a4')
    def test_backfill_allocation(self):
        self.deploy_node(self.node['uuid'])

        _, body = self.client.create_allocation(self.resource_class,
                                                node=self.node['uuid'])
        uuid = body['uuid']
        self.assertEqual(self.node['uuid'], body['node_uuid'])
        self.assertEqual('active', body['state'])
        self.assertIsNone(body['last_error'])

        _, body2 = self.client.show_node_allocation(body['node_uuid'])
        self.assertEqual(self.node['uuid'], body2['node_uuid'])
        self.assertEqual('active', body2['state'])
        self.assertIsNone(body2['last_error'])

        _, node = self.client.show_node(self.node['uuid'])
        self.assertEqual(uuid, node['allocation_uuid'])

    @decorators.idempotent_id('c33d4b65-1232-4a3f-9aad-942e32f6f7b0')
    def test_backfill_without_resource_class(self):
        self.deploy_node(self.node['uuid'])

        _, body = self.client.create_allocation(None, node=self.node['uuid'])
        uuid = body['uuid']
        self.assertEqual(self.node['uuid'], body['node_uuid'])
        self.assertEqual('active', body['state'])
        self.assertIsNone(body['last_error'])
        # Resource class is copied from node
        self.assertEqual(self.node['resource_class'], body['resource_class'])

        _, body2 = self.client.show_node_allocation(body['node_uuid'])
        self.assertEqual(self.node['uuid'], body2['node_uuid'])
        self.assertEqual('active', body2['state'])
        self.assertIsNone(body2['last_error'])
        self.assertEqual(self.node['resource_class'], body2['resource_class'])

        _, node = self.client.show_node(self.node['uuid'])
        self.assertEqual(uuid, node['allocation_uuid'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from ironic_tempest_plugin.tests.api.admin import api_microversion_fixture
from ironic_tempest_plugin.tests.api import base


class TestInspectionRules(base.BaseBaremetalTest):
    """API tests for Inspection Rules endpoints"""

    # Inspection rules API was introduced in microversion 1.96
    # We will be skipping this test for the older version.
    min_microversion = '1.96'

    def setUp(self):
        super(TestInspectionRules, self).setUp()

        _, self.node = self.create_node(None)

        self.useFixture(
            api_microversion_fixture.APIMicroversionFixture('1.96'))

    def _create_inspection_rule_payload(self, **kwargs):
        """Create a Inspection rule payload."""
        payload = {
            "description": "Inspection rule to log node UUID",
            "conditions": [],
            "actions": [
                {
                    "op": "log",
                    "args": {
                        "msg": "Node with UUID {node.uuid} is being inspected"
                    }
                }
            ],
            "phase": "main",
            "priority": 0,
            "sensitive": False
        }

        payload.update(kwargs)

        return payload

    @decorators.idempotent_id('7fb771cd-b011-409e-a255-3c71cf7251e8')
    def test_create_rule_sensitive_true(self):
        """Test creating rule with sensitive=True."""
        rule_uuid = data_utils.rand_uuid()
        payload = self._create_inspection_rule_payload(sensitive=True)

        self.create_inspection_rule(rule_uuid, payload)

        _, fetched_rule = self.client.show_inspection_rule(rule_uuid)

        self.assertTrue(fetched_rule.get('sensitive'))
        self.assertIsNone(fetched_rule.get('conditions'))
        self.assertIsNone(fetched_rule.get('actions'))

    @decorators.idempotent_id('e60b4513-7c3d-4b2c-b485-17443bf6485f')
    def test_create_rule_complex_logging_conditions_actions(self):
        """Test creating rule with loop conditions and actions"""
        complex_log_conditions = [
            {
                "op": "eq",
                "args": [
                    "{inventory.system.product_name}",
                    "{item}"
                ],
                "loop": [
                    "product_name_1",
                    "product_name_2",
                    "product_name_3"
                ],
                "multiple": "any"
            }
        ]

        complex_log_actions = [
            {
                "op": "set-attribute",
                "args": [
                    "{item[path]}",
                    "{item[value]}"
                ],
                "loop": [
                    {
                        "path": "/driver_info/ipmi_username",
                        "value": "admin"
                    },
                    {
                        "path": "/driver_info/ipmi_password",
                        "value": "password"
                    },
                    {
                        "path": "/driver_info/ipmi_address",
                        "value": "{inventory[bmc_address]}"
                    }
                ]
            }
        ]

        payload = self._create_inspection_rule_payload(
            conditions=complex_log_conditions,
            actions=complex_log_actions,
        )

        _, created_rule = self.create_inspection_rule(None, payload)

        self.assertEqual(complex_log_conditions,
                         created_rule.get('conditions'))
        self.assertEqual(complex_log_actions,
                         created_rule.get('actions'))

    @decorators.idempotent_id('a786a4ec-1e43-4fb9-8fc3-c53aa4e1f52f')
    def test_patch_conditions_actions_priority(self):
        """Test Updating rule'si priority, condition and actions"""
        payload = self._create_inspection_rule_payload()

        patch = [
            {
                "op": "replace",
                "path": "/priority",
                "value": 200
            },
            {
                "op": "replace",
                "path": "/conditions",
                "value": [
                    {
                        "op": "eq",
                        "args": ["{{ inventory.cpu.count }}", 8]
                    }
                ]
            },
            {
                "op": "replace",
                "path": "/actions",
                "value": [
                    {
                        "op": "set-attribute",
                        "args": ["{{ /properties/cpu_model }}", "cpu_xyz"]
                    },
                    {
                        "op": "log",
                        "args": ["CPU model updated via rule."]
                    }
                ]
            }
        ]

        _, created_rule = self.create_inspection_rule(None, payload)
        _, fetched_rule = self.client.update_inspection_rule(
            created_rule.get('uuid'), patch)

        self.assertEqual(fetched_rule.get('priority'),
                         patch[0]['value'])
        self.assertEqual(fetched_rule.get('conditions'),
                         patch[1]['value'])
        self.assertEqual(fetched_rule.get('actions'),
                         patch[2]['value'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import random

from oslo_utils import timeutils
from oslo_utils import uuidutils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from ironic_tempest_plugin.services.baremetal.v1.json.baremetal_client import \
    BaremetalClient
from ironic_tempest_plugin.tests.api.admin import api_microversion_fixture
from ironic_tempest_plugin.tests.api import base

CONF = config.CONF


class MicroversionTestMixin:
    """Mixin class containing shared microversion test functionality."""

    def _microversion_test(
        self,
        method_name,
        min_version,
        expected_error,
        required_args,
        ignore_positive=False,
    ):
        """Test methods with invalid API versions"""

        major, minor = map(int, min_version.split("."))
        invalid_versions = []
        if minor >= 11:
            invalid_versions.append(f"{major}.{minor - 10}")
        if minor >= 2:
            invalid_versions.append(f"{major}.{minor - 1}")
        elif minor == 0 and major > 1:
            invalid_versions.append(f"{major - 1}.99")
        elif minor == 1:
            invalid_versions.append(f"{major}.0")
        else:
            # We expect fails for v1.0 and below
            raise ValueError(f"Invalid microversion {min_version}")

        # Get method name from method object
        method_name = method_name.__name__

        # Test with invalid versions (should fail)
        for microversion in invalid_versions:
            msg = (
                f"Testing {method_name} with version {microversion} "
                f"and arguments {required_args} - should fail"
            )
            with self.subTest(
                msg=msg, method=method_name, version=microversion
            ):
                self.useFixture(
                    api_microversion_fixture.APIMicroversionFixture(
                        microversion
                    )
                )
                method = getattr(self.client, method_name)

                try:
                    method(**required_args)
                    self.fail(
                        f"Request for microversion {microversion} for "
                        f"{method_name} unexpectedly succeeded. We expected "
                        f"{expected_error.__name__}."
                    )
                except expected_error:
                    pass  # Expected error, test passes
                except Exception as e:
                    self.fail(
                        f"Request for microversion {microversion} for "
                        f"{method_name} raised unexpected exception: {e}"
                    )

        if ignore_positive:
            return True

        # Test with valid version (should succeed)
        # Use the minimum required version
        msg = (
            f"Testing {method_name} with version {min_version} "
            f"and arguments {required_args} - should succeed"
        )
        with self.subTest(
            msg=msg, method=method_name, version=min_version
        ):
            self.useFixture(
                api_microversion_fixture.APIMicroversionFixture(
                    min_version
                )
            )
            method = getattr(self.client, method_name)

            try:
                # We don't check the actual response, just
                # that it doesn't raise
                # the expected error from the negative test
                method(**required_args)
                return True

            except expected_error as e:
                self.fail(
                    f"Method {method_name} failed with valid "
                    f"microversion {min_version}: {e}"
                )
            except Exception as e:
                # Other exceptions might be expected due to invalid test data
                # For example, a 404 might be expected if we're using fake IDs
                self.assertNotIsInstance(
                    e,
                    expected_error,
                    (
                        f"Got unexpected {expected_error.__name__} with valid "
                        f"microversion {min_version}: {e}"
                    ),
                )


class BaseTestMicroversionEnforcement(base.BaseBaremetalTest):
    """Base class for microversion enforcement tests."""

    def setUp(self):
        super().setUp()
        self.resource_class = uuidutils.generate_uuid()


class TestNodesMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for nodes-related API microversion enforcement."""

    min_microversion = "1.1"

    def setUp(self):
        super(TestNodesMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()

    # NOTE(adamcarthur) - We test the positive case in the other
    # tests. Doing it here would require changing the test function
    # to allow returns.
    @decorators.idempotent_id("be29d566-43d5-41ce-8c8a-ce360af9995b")
    def test_create_node(self):
        self._microversion_test(
            BaremetalClient.create_node,
            "1.1",
            lib_exc.NotAcceptable,
            {"chassis_id": self.chassis["uuid"]},
            ignore_positive=True
        )

    @decorators.idempotent_id("3ddd0ae9-979b-44ac-9cdc-2cabd2f61118")
    def test_list_nodes(self):
        self._microversion_test(
            BaremetalClient.list_nodes, "1.1", lib_exc.NotAcceptable, {}
        )

    @decorators.idempotent_id("d47160dd-654e-427b-91b0-9a4298a7ff54")
    def test_list_nodes_detail(self):
        self._microversion_test(
            BaremetalClient.list_nodes_detail, "1.1", lib_exc.NotAcceptable, {}
        )

    @decorators.idempotent_id("a612b489-d4c6-4f29-9dcd-ca7a71d93f77")
    def test_show_node(self):
        _, node = self.create_node(self.chassis["uuid"])
        self._microversion_test(
            BaremetalClient.show_node,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": node["uuid"]},
        )
        # delete the node
        self.client.delete_node(node["uuid"])

    @decorators.idempotent_id("02623737-84f5-42f1-a8f3-0f3a8c1319da")
    def test_update_node(self):
        _, node = self.create_node(self.chassis["uuid"])
        instance_uuid = data_utils.rand_uuid()
        self._microversion_test(
            BaremetalClient.update_node,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": node["uuid"], "patch": instance_uuid},
        )
        # delete the node
        self.client.delete_node(node["uuid"])

    @decorators.idempotent_id("59f433f1-05b8-47a9-b14a-f4aba337698d")
    def test_delete_node(self):
        _, node = self.create_node(self.chassis["uuid"])
        self._microversion_test(
            BaremetalClient.delete_node,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": node["uuid"]},
        )


class TestNodeManagementMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node management-related API microversion enforcement."""

    min_microversion = "1.1"

    def setUp(self):
        super(TestNodeManagementMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    def _validate_provision_state(self, node_uuid, target_state):
        # Validate that provision state is set within timeout
        start = timeutils.utcnow()
        while timeutils.delta_seconds(
                start, timeutils.utcnow()) < self.unprovision_timeout:
            _, node = self.client.show_node(node_uuid)
            if node['provision_state'] == target_state:
                return
        message = ('Failed to set provision state %(state)s within '
                   'the required time: %(timeout)s sec.',
                   {'state': target_state,
                    'timeout': self.unprovision_timeout})
        raise lib_exc.TimeoutException(message)

    @decorators.idempotent_id("54a58e1e-8334-4152-84b4-2974cabb74e8")
    def test_validate_driver_interface(self):
        self._microversion_test(
            BaremetalClient.validate_driver_interface,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"]},
        )

    @decorators.idempotent_id("9adfa595-8937-4a99-83e5-6012ddcbddc0")
    def test_set_node_power_state(self):
        self._microversion_test(
            BaremetalClient.set_node_power_state,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"], "state": "power off"},
        )

    @decorators.idempotent_id("efd978ce-2ce5-4dc3-8f69-ff063644a550")
    def test_set_node_provision_state(self):
        contd = self._microversion_test(
            BaremetalClient.set_node_provision_state,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"], "state": "manageable"},
        )

        if contd:
            self._validate_provision_state(self.node["uuid"], "manageable")

    @decorators.idempotent_id("893e4815-2c55-40f6-84e3-1816e28f6803")
    def test_set_node_raid_config(self):
        self._microversion_test(
            BaremetalClient.set_node_raid_config,
            "1.12",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"], "target_raid_config": {}},
        )

    @decorators.idempotent_id("f6b835da-dea3-46b2-8e16-6f910ad7cc86")
    def test_get_console(self):
        self._microversion_test(
            BaremetalClient.get_console,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"]},
        )

    @decorators.idempotent_id("2dc96b6b-5713-467d-9707-389292ae5460")
    def test_set_console_mode(self):
        self._microversion_test(
            BaremetalClient.set_console_mode,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"], "enabled": True},
        )

    @decorators.idempotent_id("4b4e23ae-3d18-4f35-a140-d10abcf22110")
    def test_set_node_boot_device(self):
        self._microversion_test(
            BaremetalClient.set_node_boot_device,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"], "boot_device": "pxe"},
        )

    @decorators.idempotent_id("f13b398e-cde1-4862-a183-f20eb56d3234")
    def test_get_node_boot_device(self):
        self._microversion_test(
            BaremetalClient.get_node_boot_device,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"]},
        )

    @decorators.idempotent_id("88985d9e-ccd1-4894-bc5d-e712ce29b892")
    def test_get_node_supported_boot_devices(self):
        self._microversion_test(
            BaremetalClient.get_node_supported_boot_devices,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"]},
        )


class TestNodeVMediaMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node vmedia-related API microversion enforcement."""

    min_microversion = "1.89"

    def setUp(self):
        super(TestNodeVMediaMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("305c0689-c43b-407c-8b78-932ff86e4b3d")
    def test_attach_node_vmedia(self):
        self._microversion_test(
            BaremetalClient._put_request,
            "1.89",
            lib_exc.NotFound,
            {
                "resource": f"nodes/{self.node['uuid']}/vmedia",
                "put_object": {
                    "device_type": "CDROM", "image_url": "http://image"
                },
            },
        )

    @decorators.idempotent_id("1d4d15bb-a66c-47f9-bf41-cc5e09854bcc")
    def test_detach_node_vmedia(self):
        self._microversion_test(
            BaremetalClient._delete_request,
            "1.89",
            lib_exc.NotFound,
            {"resource": f"nodes/{self.node['uuid']}/vmedia", "uuid": ""},
        )


class TestNodeVendorPassthruMethodsMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node vendor passthru methods API microversion enforcement."""

    min_microversion = "1.1"

    def setUp(self):
        super(TestNodeVendorPassthruMethodsMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("8599e266-debc-4548-af5a-cd2223a5e051")
    def test_list_vendor_passthru_methods(self):
        self._microversion_test(
            BaremetalClient.list_vendor_passthru_methods,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"]},
        )


class TestNodeVendorPassthruCallMethodMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for vendor passthru call method API microversion"""

    min_microversion = "1.1"

    def setUp(self):
        super(TestNodeVendorPassthruCallMethodMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("35b34e5d-78d4-49e6-81b3-48b536f9fb60")
    def test_call_vendor_passthru_method(self):
        self._microversion_test(
            BaremetalClient._put_request,
            "1.1",
            lib_exc.NotAcceptable,
            {
                "resource": f"nodes/{self.node['uuid']}/vendor_passthru",
                "put_object": {"method": "fakemethod"},
            },
        )


class TestNodeTraitsMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node traits-related API microversion enforcement."""

    min_microversion = "1.37"

    def setUp(self):
        super(TestNodeTraitsMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])
        self.traits = ["CUSTOM_TRAIT1", "HW_CPU_X86_VMX"]

    @decorators.idempotent_id("3f4ba6c1-5b39-4cb1-a823-21dcea802d61")
    def test_list_node_traits(self):
        self._microversion_test(
            BaremetalClient.list_node_traits,
            "1.37",
            lib_exc.NotAcceptable,
            {"node_uuid": self.node["uuid"]},
        )

    @decorators.idempotent_id("10d77871-cd8a-4ecb-b6a4-f38d6ad719fe")
    def test_set_node_traits(self):
        self._microversion_test(
            BaremetalClient.set_node_traits,
            "1.37",
            lib_exc.UnexpectedResponseCode,
            {"node_uuid": self.node["uuid"], "traits": self.traits},
        )

    @decorators.idempotent_id("12ca1f62-9d4a-490c-8f8a-a10fc742630a")
    def test_add_node_trait(self):
        self._microversion_test(
            BaremetalClient.add_node_trait,
            "1.37",
            lib_exc.UnexpectedResponseCode,
            {"node_uuid": self.node["uuid"], "trait": "CUSTOM_TRAIT1"},
        )

    @decorators.idempotent_id("dd2f3b89-3fe3-4861-ba11-608b365ff484")
    def test_remove_node_traits(self):
        # add a trait to remove it
        self.client.add_node_trait(self.node["uuid"], "CUSTOM_TRAIT1")
        self._microversion_test(
            BaremetalClient.remove_node_traits,
            "1.37",
            lib_exc.NotFound,
            {"node_uuid": self.node["uuid"]},
        )

    @decorators.idempotent_id("be438903-26d0-4ee6-970e-2d6d66058842")
    def test_remove_node_trait(self):
        self.client.add_node_trait(self.node["uuid"], "CUSTOM_TRAIT1")
        self._microversion_test(
            BaremetalClient.remove_node_trait,
            "1.37",
            lib_exc.NotFound,
            {"node_uuid": self.node["uuid"], "trait": "CUSTOM_TRAIT1"},
        )


class TestNodeVIFsMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node VIFs-related API microversion enforcement."""

    min_microversion = "1.28"

    def setUp(self):
        super(TestNodeVIFsMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])
        self.vif_id = uuidutils.generate_uuid()

    @decorators.idempotent_id("9b26c364-0623-4bd4-ac53-a3eca8d58fea")
    def test_list_node_vifs(self):
        self._microversion_test(
            BaremetalClient.vif_list,
            "1.28",
            lib_exc.NotFound,
            {"node_uuid": self.node["uuid"]},
        )

    @decorators.idempotent_id("4e91a305-145d-408d-9aea-08d73ef8d082")
    def test_attach_node_vif(self):
        self._microversion_test(
            BaremetalClient.vif_attach,
            "1.28",
            lib_exc.NotFound,
            {"node_uuid": self.node["uuid"], "vif_id": self.vif_id},
        )

    @decorators.idempotent_id("ce93713b-5df8-44ce-815c-c18ae0203c63")
    def test_detach_node_vif(self):
        self._microversion_test(
            BaremetalClient.vif_detach,
            "1.28",
            lib_exc.NotFound,
            {"node_uuid": self.node["uuid"], "vif_id": self.vif_id},
        )


class TestNodeIndicatorsMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node indicators-related API microversion enforcement."""

    min_microversion = "1.1"

    def setUp(self):
        super(TestNodeIndicatorsMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])
        self.component = "system"
        self.ind_ident = "led"
        self.state = "ON"

    @decorators.idempotent_id("79194e54-c98d-4e55-a368-2a2ba2199844")
    def test_get_node_indicator_state(self):
        self._microversion_test(
            BaremetalClient.get_node_indicator_state,
            "1.1",
            lib_exc.NotAcceptable,
            {
                "node_uuid": self.node["uuid"],
                "component": self.component,
                "ind_ident": self.ind_ident,
            },
        )


class TestPortgroupMicroversions(BaseTestMicroversionEnforcement,
                                 MicroversionTestMixin):
    """Tests for Portgroup-related API microversion enforcement.

    Portgroup APIs (e.g. listing, creating, showing, updating and deleting)
    were introduced in microversion 1.23.
    """

    min_microversion = "1.23"

    def setUp(self):
        super(TestPortgroupMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis['uuid'])

    @decorators.idempotent_id("a8b52cdf-8471-40d7-9bc0-6ecfefa33adf")
    def test_list_portgroups(self):
        self._microversion_test(
            BaremetalClient.list_portgroups,
            "1.23",
            lib_exc.NotFound,
            {}  # Empty dict for no required args
        )

    @decorators.idempotent_id("cb5b92f0-fdb6-4789-b9b5-ae147cbe288b")
    def test_list_portgroups_detail(self):
        self._microversion_test(
            BaremetalClient.list_portgroups_detail,
            "1.23",
            lib_exc.NotFound,
            {}  # Empty dict for no required args
        )

    @decorators.idempotent_id("aafb5f2d-22e4-4b07-98c6-7f8f5f4a6699")
    def test_create_portgroup(self):
        self._microversion_test(
            BaremetalClient.create_portgroup,
            "1.23",
            lib_exc.NotFound,
            {
                "node_uuid": self.node['uuid'],
                "address": "11:11:11:11:11:11",
                "name": "test_portgroup"
            }
        )

    @decorators.idempotent_id("ea27635c-132f-4cff-9f93-b11bd4baa5a2")
    def test_show_portgroup(self):
        _, portgroup = self.client.create_portgroup(self.node['uuid'])
        self._microversion_test(
            BaremetalClient.show_portgroup,
            "1.23",
            lib_exc.NotFound,
            {"portgroup_ident": portgroup['uuid']}
        )

    @decorators.idempotent_id("48dfef1f-b95f-4767-86ee-d6257c52cefd")
    def test_update_portgroup(self):
        patch_doc = [
            {"op": "replace", "path": "/address", "value": "22:22:22:22:22:22"}
        ]
        # Need to create a portgroup to delete it
        _, portgroup = self.create_portgroup(self.node['uuid'])
        self._microversion_test(
            BaremetalClient.update_portgroup,
            "1.23",
            lib_exc.NotFound,
            {
                "uuid": portgroup['uuid'],
                "patch": patch_doc
            }
        )

    @decorators.idempotent_id("e0884cbe-bacc-4347-be14-72e475855eff")
    def test_delete_portgroup(self):
        # Need to create a portgroup to delete it
        _, portgroup = self.create_portgroup(self.node['uuid'])

        self._microversion_test(
            BaremetalClient.delete_portgroup,
            "1.23",
            lib_exc.NotFound,
            {"portgroup_ident": portgroup['uuid']}
        )


class TestPortgroupByNodeMicroversions(BaseTestMicroversionEnforcement,
                                       MicroversionTestMixin):
    """Tests for listing Portgroups by Node.

    These endpoints (e.g. /v1/nodes/{node_ident}/portgroups and
    /v1/nodes/{node_ident}/portgroups/detail) were introduced
    in microversion 1.24.
    """

    min_microversion = "1.24"

    def setUp(self):
        super(TestPortgroupByNodeMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis['uuid'])

    @decorators.idempotent_id("c967b02f-1acc-4714-98b0-07e49e9b3482")
    def test_list_portgroups_by_node(self):
        self._microversion_test(
            BaremetalClient.list_portgroups_by_node,
            "1.24",
            lib_exc.NotFound,
            {"node_ident": self.node['uuid']}
        )

    @decorators.idempotent_id("8c38415b-1aad-4519-81fb-0792ebf3731a")
    def test_list_portgroups_details_by_node(self):
        self._microversion_test(
            BaremetalClient.list_portgroups_details_by_node,
            "1.24",
            lib_exc.NotFound,
            {"node_ident": self.node['uuid']}
        )


class TestPortMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for port-related API microversion enforcement."""

    min_microversion = "1.1"

    def setUp(self):
        super(TestPortMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])
        random_mac = (
            f"02:00:00:{random.randint(0, 255):02x}"
            f":{random.randint(0, 255):02x}"
            f":{random.randint(0, 255):02x}"
        )

        _, self.port = self.create_port(
            self.node["uuid"], address=random_mac
        )

    @decorators.idempotent_id("99e628a4-0d10-44a1-aed5-0b82d0155103")
    def test_list_ports(self):
        self._microversion_test(
            BaremetalClient.list_ports,
            "1.1",
            lib_exc.NotAcceptable,
            {},  # Empty dict for no required args
        )

    @decorators.idempotent_id("0c01ce9a-8945-4604-b315-3f8a1856c621")
    def test_list_ports_detail(self):
        self._microversion_test(
            BaremetalClient.list_ports_detail,
            "1.1",
            lib_exc.NotAcceptable,
            {},  # Empty dict for no required args
        )

    @decorators.idempotent_id("15b3f4d5-d8ca-44af-b0fe-75f9a86896b1")
    def test_create_port(self):
        self._microversion_test(
            BaremetalClient.create_port,
            "1.1",
            lib_exc.NotAcceptable,
            {
                "node_id": self.node["uuid"],
                "address": "11:11:11:11:11:11",
            },
        )

    @decorators.idempotent_id("637f088f-1460-46c4-a3cf-3568c1997461")
    def test_show_port(self):
        self._microversion_test(
            BaremetalClient.show_port,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": "fake_port"},
        )

    @decorators.idempotent_id("5cf7930d-3e0b-44f7-b9b8-a941dea2782e")
    def test_update_port(self):
        patch_doc = [
            {"op": "replace", "path": "/address", "value": "22:22:22:22:22:22"}
        ]
        self._microversion_test(
            BaremetalClient.update_port,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": self.port["uuid"], "patch": patch_doc},
        )

    @decorators.idempotent_id("b5d92c53-9d2e-485f-ac6c-1e0d020d5af3")
    def test_delete_port(self):
        self._microversion_test(
            BaremetalClient.delete_port,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": self.port["uuid"]},
        )


class TestNodePortsMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for listing Ports by Node API microversion enforcement."""

    min_microversion = "1.1"

    def setUp(self):
        super(TestNodePortsMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("7a0acf09-d527-4d77-92bd-3eac81687bc7")
    def test_list_ports_by_node(self):
        self._microversion_test(
            BaremetalClient.list_node_ports,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": self.node["uuid"]},
        )

    @decorators.idempotent_id("df017231-a8d3-4b6b-8583-0e6fbf61e834")
    def test_list_ports_details_by_node(self):
        self._microversion_test(
            BaremetalClient.list_node_ports,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": self.node["uuid"]},
        )


class TestPortgroupPortsMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for listing Ports by Portgroup API microversion enforcement."""

    min_microversion = "1.24"

    def setUp(self):
        super(TestPortgroupPortsMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])
        _, self.portgroup = self.create_portgroup(self.node["uuid"])

    @decorators.idempotent_id("bb3a1fa6-52cf-4e19-98d0-746c2421ff9a")
    def test_list_ports_by_portgroup(self):
        self._microversion_test(
            BaremetalClient.list_ports,
            "1.24",
            lib_exc.NotAcceptable,
            {"portgroup": self.portgroup["uuid"]},
        )

    @decorators.idempotent_id("6740fe09-539d-415b-bc44-f38f458f2547")
    def test_list_ports_details_by_portgroup(self):
        self._microversion_test(
            BaremetalClient.list_ports_detail,
            "1.24",
            lib_exc.NotAcceptable,
            {"portgroup": self.portgroup["uuid"]},
        )


class TestVolumeConnectorMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for volume connector-related API microversion enforcement."""

    min_microversion = "1.32"

    def setUp(self):
        super(TestVolumeConnectorMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("1bde0eb4-c63c-4d05-897d-b2247dd97863")
    def test_list_volume_connectors(self):
        self._microversion_test(
            BaremetalClient.list_volume_connectors,
            "1.32",
            lib_exc.NotFound,
            {},
        )

    @decorators.idempotent_id("7673b980-b882-45d7-bc89-8e255e2228cc")
    def test_show_volume_connector(self):
        self._microversion_test(
            BaremetalClient.show_volume_connector,
            "1.32",
            lib_exc.NotFound,
            {"volume_connector_ident": "fake_volume_connector"},
        )

    @decorators.idempotent_id("3e7d6a89-5302-4087-a10a-200b8d9dc026")
    def test_create_volume_connector(self):
        self._microversion_test(
            BaremetalClient.create_volume_connector,
            "1.32",
            lib_exc.NotFound,
            {
                "node_uuid": self.node["uuid"],
                "type": "iqn",
                "connector_id": "fake_connector_id",
            },
        )

    @decorators.idempotent_id("cca4b764-12cc-4d4d-a702-0b52ef1b0e94")
    def test_update_volume_connector(self):
        patch = [{"op": "replace", "path": "/connector_id", "value": "new_id"}]
        self._microversion_test(
            BaremetalClient.update_volume_connector,
            "1.32",
            lib_exc.NotFound,
            {"uuid": "fake_volume_connector", "patch": patch},
        )

    @decorators.idempotent_id("e05e85d1-da6b-43b6-adc4-5e4bc8e4c72d")
    def test_delete_volume_connector(self):
        self._microversion_test(
            BaremetalClient.delete_volume_connector,
            "1.32",
            lib_exc.NotFound,
            {"volume_connector_ident": "fake_volume_connector"},
        )


class TestVolumeTargetMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for volume target-related API microversion enforcement."""

    min_microversion = "1.32"

    def setUp(self):
        super(TestVolumeTargetMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("c7511a8a-4d6b-4439-bfb1-c659808c0967")
    def test_list_volume_targets(self):
        self._microversion_test(
            BaremetalClient.list_volume_targets, "1.32", lib_exc.NotFound, {}
        )

    @decorators.idempotent_id("ddb8223e-2f63-4a74-aa2f-229a522ae3e8")
    def test_show_volume_target(self):
        self._microversion_test(
            BaremetalClient.show_volume_target,
            "1.32",
            lib_exc.NotFound,
            {"volume_target_ident": "fake_volume_target"},
        )

    @decorators.idempotent_id("3fb1e9c5-8154-4308-a765-393ab80762c8")
    def test_create_volume_target(self):
        self._microversion_test(
            BaremetalClient.create_volume_target,
            "1.32",
            lib_exc.NotFound,
            {
                "node_uuid": self.node["uuid"],
                "volume_type": "iscsi",
                "volume_id": "fake_volume_id",
                "boot_index": 0,
            },
        )

    @decorators.idempotent_id("16d83cd1-0425-4e28-bfd4-33199e5a87c7")
    def test_update_volume_target(self):
        patch = [{"op": "replace", "path": "/volume_id", "value": "new_id"}]
        self._microversion_test(
            BaremetalClient.update_volume_target,
            "1.32",
            lib_exc.NotFound,
            {"uuid": "fake_volume_target", "patch": patch},
        )

    @decorators.idempotent_id("697b67be-ccc5-4a7b-99ca-276e5e8c931a")
    def test_delete_volume_target(self):
        self._microversion_test(
            BaremetalClient.delete_volume_target,
            "1.32",
            lib_exc.NotFound,
            {"volume_target_ident": "fake_volume_target"},
        )


class TestNodeVolumeMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node volume-related API microversion enforcement."""

    min_microversion = "1.32"

    def setUp(self):
        super(TestNodeVolumeMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("2ff700e6-4914-4eed-8342-308a6f2134a5")
    def test_list_node_volume(self):
        self._microversion_test(
            BaremetalClient._list_request,
            "1.32",
            lib_exc.NotFound,
            {"resource": f"nodes/{self.node['uuid']}/volume"},
        )

    @decorators.idempotent_id("3e56f64f-6bac-4126-ba7d-d89275318975")
    def test_list_node_volume_connectors(self):
        self._microversion_test(
            BaremetalClient._list_request,
            "1.32",
            lib_exc.NotFound,
            {"resource": f"nodes/{self.node['uuid']}/volume/connectors"},
        )

    @decorators.idempotent_id("060008fe-d478-49d2-bdf0-d327f1e360a5")
    def test_list_node_volume_targets(self):
        self._microversion_test(
            BaremetalClient._list_request,
            "1.32",
            lib_exc.NotFound,
            {"resource": f"nodes/{self.node['uuid']}/volume/targets"},
        )


class TestDriverMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for driver-related API microversion enforcement."""

    min_microversion = "1.1"

    @decorators.idempotent_id("c5f179c9-3b8a-49c6-bee0-734979d544bc")
    def test_list_drivers(self):
        self._microversion_test(
            BaremetalClient.list_drivers, "1.1", lib_exc.NotAcceptable, {}
        )

    @decorators.idempotent_id("f79931be-dc7b-4e60-89ce-90a4e2793714")
    def test_show_driver(self):
        self._microversion_test(
            BaremetalClient.show_driver,
            "1.1",
            lib_exc.NotAcceptable,
            {"driver_name": "testdriver"},
        )

    @decorators.idempotent_id("daef2f42-e343-4640-a5d4-d8938e956ddb")
    def test_show_driver_properties(self):
        self._microversion_test(
            BaremetalClient.get_driver_properties,
            "1.1",
            lib_exc.NotAcceptable,
            {"driver_name": "testdriver"},
        )

    @decorators.idempotent_id("bade310a-0699-4470-8da3-6f8ae26e9d71")
    def test_show_driver_logical_disk_properties(self):
        self._microversion_test(
            BaremetalClient.get_driver_logical_disk_properties,
            "1.12",
            lib_exc.NotAcceptable,
            {"driver_name": "testdriver"},
        )


class TestDriverVendorPassthruMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for driver vendor passthru API microversion enforcement."""

    min_microversion = "1.1"

    @decorators.idempotent_id("27c0157d-758a-40c9-b0fd-bde05ed8a6fc")
    def test_list_vendor_passthru_methods(self):
        self._microversion_test(
            BaremetalClient.list_vendor_passthru_methods,
            "1.1",
            lib_exc.NotAcceptable,
            {"node_uuid": "fake_node"},
        )


class TestNodeBiosMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node bios-related API microversion enforcement."""

    min_microversion = "1.40"

    def setUp(self):
        super(TestNodeBiosMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("126de288-71f9-4e4f-8591-6ddfc9ac3095")
    def test_list_node_bios(self):
        self._microversion_test(
            BaremetalClient.list_node_bios_settings,
            "1.40",
            lib_exc.NotFound,
            {"uuid": self.node["uuid"]},
        )


class TestNodeBiosDetailsMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node bios details API microversion enforcement."""

    min_microversion = "1.40"

    def setUp(self):
        super(TestNodeBiosDetailsMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("68583cd5-63d0-4bda-a836-9bda6d5b2ab4")
    def test_list_node_bios_details(self):
        self._microversion_test(
            BaremetalClient.list_node_bios_settings,
            "1.40",
            lib_exc.NotFound,
            {"uuid": self.node["uuid"]},
        )


class TestNodeFirmwarenMicroversions(
        BaseTestMicroversionEnforcement,
        MicroversionTestMixin):

    min_microversion = "1.86"

    def setUp(self):
        super(TestNodeFirmwarenMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis['uuid'])

    @decorators.idempotent_id('f50e9098-1870-46b1-b05c-660d0f8c534d')
    def test_list_node_firmware(self):
        self._microversion_test(
            BaremetalClient.list_node_firmware, "1.86",
            lib_exc.NotFound, {"node_uuid": self.node['uuid']}
        )


class TestConductorMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):

    min_microversion = "1.49"

    def setUp(self):
        super(TestConductorMicroversions, self).setUp()

    @decorators.idempotent_id("412fde59-afe7-442d-83c9-538c87b95687")
    def test_list_conductors(self):
        self._microversion_test(
            BaremetalClient.list_conductors, "1.49", lib_exc.NotFound, {}
        )

    @decorators.idempotent_id("ad496ff6-2d6e-4e63-9c64-60c5184a18a7")
    def test_show_conductor(self):
        _, conductors = self.client.list_conductors()
        self.assertTrue(len(conductors['conductors']) > 0)
        conductor = conductors['conductors'].pop()

        _, conductor = self.client.show_conductor(conductor['hostname'])
        self._microversion_test(
            BaremetalClient.show_conductor,
            "1.49",
            lib_exc.NotFound,
            {"hostname": conductor['hostname']},
        )


class TestAllocationMicroversions(
        BaseTestMicroversionEnforcement,
        MicroversionTestMixin):
    """Tests for allocation-related API microversion enforcement."""

    min_microversion = "1.52"

    @decorators.idempotent_id('ef7bd40a-c1ef-4d40-a0b6-f388d23943c3')
    def test_create_allocations(self):
        self._microversion_test(
            BaremetalClient.create_allocation, "1.52",
            lib_exc.UnexpectedResponseCode, {
                "resource_class": self.resource_class
            }
        )

    @decorators.idempotent_id('511e0c4b-1320-4ac5-9c4a-fb0394d3ff67')
    def test_list_allocations(self):
        self._microversion_test(
            BaremetalClient.list_allocations, "1.52",
            lib_exc.NotFound, {}
        )

    @decorators.idempotent_id('a0d17f90-baa0-4518-95f7-a7eab73ff6d1')
    def test_show_allocations(self):
        _, allocation = self.create_allocation(self.resource_class)

        self._microversion_test(
            BaremetalClient.show_allocation, "1.52",
            lib_exc.NotFound, {
                "allocation_ident": allocation['uuid']
            }
        )

    @decorators.idempotent_id('b05a9b1a-4a12-4b55-93c7-530c3f35c7d9')
    def test_delete_allocations(self):
        _, allocation = self.create_allocation(self.resource_class)

        self._microversion_test(
            BaremetalClient.delete_allocation, "1.52",
            lib_exc.UnexpectedResponseCode, {
                "allocation_ident": allocation['uuid']
            }
        )


class TestNodeAllocationMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node allocation-related API microversion enforcement."""

    min_microversion = "1.52"

    def setUp(self):
        super(TestNodeAllocationMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("9b170cac-18f7-4a17-be2e-314b548244ba")
    def test_show_allocation_by_node(self):
        _, body = self.create_allocation(self.resource_class)

        self._microversion_test(
            BaremetalClient.show_node_allocation,
            "1.52",
            lib_exc.NotFound,
            {"node_ident": body["node_uuid"]},
        )


class TestDeployTemplateMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for deploy template-related API microversion enforcement."""

    min_microversion = "1.55"

    def setUp(self):
        super(TestDeployTemplateMicroversions, self).setUp()
        self.deploy_template_name = "CUSTOM_TEMPLATE_123"
        self.steps = [
            {
                "interface": "bios",
                "step": "apply_configuration",
                "args": {
                    "settings": [
                        {"name": "LogicalProc", "value": "Enabled"}
                    ]
                },
                "priority": 150,
            }
        ]

    @decorators.idempotent_id("52806f1d-d64a-4b7f-a799-e623ecc09bda")
    def test_create_deploy_template(self):
        self._microversion_test(
            BaremetalClient.create_deploy_template,
            "1.55",
            lib_exc.UnexpectedResponseCode,
            {"name": self.deploy_template_name, "steps": self.steps},
        )

    @decorators.idempotent_id("1a3bb543-faa9-4da6-a1c4-20cb8177f53d")
    def test_list_deploy_templates(self):
        self._microversion_test(
            BaremetalClient.list_deploy_templates, "1.55", lib_exc.NotFound, {}
        )

    @decorators.idempotent_id("a0933551-8569-4638-acd4-fad0a9daefd9")
    def test_show_deploy_template(self):
        _, deploy_template = self.client.create_deploy_template(
            "CUSTOM_TEMPLATE_456", steps=self.steps
        )
        self._microversion_test(
            BaremetalClient.show_deploy_template,
            "1.55",
            lib_exc.NotFound,
            {"deploy_template_ident": deploy_template["uuid"]},
        )

    @decorators.idempotent_id("f47a12a7-71b7-4ff0-ab17-9f45c9ac68e7")
    def test_update_deploy_template(self):
        patch = [{"op": "replace", "path": "/name", "value": "new_name"}]
        self._microversion_test(
            BaremetalClient.update_deploy_template,
            "1.55",
            lib_exc.UnexpectedResponseCode,
            {"deploy_template_ident": "fakedt", "patch": patch},
        )

    @decorators.idempotent_id("b987a887-22ca-4ec9-a074-0a9bd3eac7ac")
    def test_delete_deploy_template(self):
        self._microversion_test(
            BaremetalClient.delete_deploy_template,
            "1.55",
            lib_exc.UnexpectedResponseCode,
            {"deploy_template_ident": "fakedt"},
        )


class TestRunbookMicroversions(BaseTestMicroversionEnforcement,
                               MicroversionTestMixin):
    """Tests for runbook-related API microversion enforcement.

    The runbook API was introduced in microversion 1.92. Operations such as
    creating, updating, and deleting runbooks are expected to fail with an
    error if used with an unsupported microversion.
    """

    min_microversion = "1.92"

    @decorators.idempotent_id("e37be789-fcdf-435a-8a16-3d2f8f21e6e2")
    def test_create_runbook(self):
        # Define a minimal valid runbook payload.
        steps = [
            {
                "interface": "bios",
                "step": "apply_configuration",
                "order": 1,
                "args": {
                    "settings": [
                        {"name": "LogicalProc", "value": "Enabled"}
                    ]
                }
            }
        ]
        self._microversion_test(
            BaremetalClient.create_runbook,
            "1.92",
            lib_exc.UnexpectedResponseCode,
            {"name": "CUSTOM_TEST", "steps": steps}
        )

    @decorators.idempotent_id("f1b5b500-6b9c-41f6-bb99-3a77e3bd6ac4")
    def test_list_runbooks(self):
        self._microversion_test(
            BaremetalClient.list_runbooks,
            "1.92",
            lib_exc.NotFound,
            {}
        )

    @decorators.idempotent_id("3d12345a-3d56-4fbb-afd2-7bd0ebc32e19")
    def test_show_runbook(self):
        steps = [
            {
                "interface": "bios",
                "step": "apply_configuration",
                "order": 1,
                "args": {
                    "settings": [
                        {"name": "LogicalProc", "value": "Enabled"}
                    ]
                }
            }
        ]
        _, runbook = self.client.create_runbook("CUSTOM_TEST_2", steps=steps)
        self._microversion_test(
            BaremetalClient.show_runbook,
            "1.92",
            lib_exc.NotFound,
            {"runbook_ident": runbook['uuid']}
        )

    @decorators.idempotent_id("a1e4cd23-56be-4b39-b756-9a9b73f21492")
    def test_update_runbook(self):
        # Using an example JSON PATCH document to update the runbook.
        patch_doc = [
            {"op": "replace", "path": "/name", "value": "CUSTOM_TEST2"}
        ]
        self._microversion_test(
            BaremetalClient.update_runbook,
            "1.92",
            lib_exc.UnexpectedResponseCode,
            {"runbook_ident": "fake_runbook", "patch": patch_doc}
        )

    @decorators.idempotent_id("cb9cf9a6-2a3f-4aa7-9488-098f8aeeb5d9")
    def test_delete_runbook(self):
        self._microversion_test(
            BaremetalClient.delete_runbook,
            "1.92",
            lib_exc.UnexpectedResponseCode,
            {"runbook_ident": "fake_runbook"}
        )


class TestNodeHistoryMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node history-related API microversion enforcement."""

    min_microversion = "1.78"

    def setUp(self):
        super(TestNodeHistoryMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("aea6cb45-6519-44d9-a33e-aab7cd72d769")
    def test_list_node_history(self):
        self._microversion_test(
            BaremetalClient.list_node_history,
            "1.78",
            lib_exc.NotFound,
            {"node_uuid": self.node["uuid"]},
        )


# NOTE(adamcarthur) - The positive test for this will need a configured
# node to have history.
class TestNodeInventoryMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for node inventory-related API microversion enforcement."""

    min_microversion = "1.81"

    def setUp(self):
        super(TestNodeInventoryMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])

    @decorators.idempotent_id("ca557dca-9b3e-44cb-a101-f31d58a0dedd")
    def test_show_node_inventory(self):
        _, node = self.client.show_node(self.node['uuid'])
        self._microversion_test(
            BaremetalClient.show_inventory,
            "1.81",
            lib_exc.NotFound,
            {"uuid": node["uuid"]},
            ignore_positive=True
        )


class TestShardMicroversions(
        BaseTestMicroversionEnforcement,
        MicroversionTestMixin):
    """Tests for shard-related API microversion enforcement."""

    min_microversion = "1.82"

    @decorators.idempotent_id("e5403a31-e12b-4f97-a776-dcb819e5e9a0")
    def test_shard(self):
        self._microversion_test(
            BaremetalClient.get_shards, "1.82", lib_exc.NotFound, {}
        )

    @decorators.idempotent_id("5df533c6-7a9c-4639-a47f-1377a2a87e6a")
    def test_list_node_filter_shard(self):
        self._microversion_test(
            BaremetalClient.list_nodes, "1.82",
            lib_exc.NotAcceptable, {"shard": "testshard"}
        )


class TestChassisMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for chassis-related API microversion enforcement."""

    min_microversion = "1.1"

    def setUp(self):
        super(TestChassisMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()

    @decorators.idempotent_id("59c20dbf-280e-4148-ac8e-be3b8d75d021")
    def test_create_chassis(self):
        self._microversion_test(
            BaremetalClient.create_chassis, "1.1", lib_exc.NotAcceptable, {}
        )

    @decorators.idempotent_id("0e840e17-5659-4ad9-a218-84b14bc8b013")
    def test_list_chassis(self):
        self._microversion_test(
            BaremetalClient.list_chassis, "1.1", lib_exc.NotAcceptable, {}
        )

    @decorators.idempotent_id("6c3a5728-0d96-41aa-a3ca-295bfbbe341e")
    def test_show_chassis(self):
        self._microversion_test(
            BaremetalClient.show_chassis,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": self.chassis["uuid"]},
        )

    @decorators.idempotent_id("7ae8ca94-b7d9-4975-87c6-37d89779b8b5")
    def test_update_chassis(self):
        patch = [
            {"op": "replace", "path": "/description", "value": "new_desc"}
        ]
        self._microversion_test(
            BaremetalClient.update_chassis,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": self.chassis["uuid"], "patch": patch},
        )

    @decorators.idempotent_id("f6d70a3a-9212-46af-bb7b-c95a3043a722")
    def test_delete_chassis(self):
        self._microversion_test(
            BaremetalClient.delete_chassis,
            "1.1",
            lib_exc.NotAcceptable,
            {"uuid": self.chassis["uuid"]},
        )


class TestUtilityMicroversions(
    BaseTestMicroversionEnforcement, MicroversionTestMixin
):
    """Tests for utility-related API microversion enforcement."""

    min_microversion = "1.22"

    def setUp(self):
        super(TestUtilityMicroversions, self).setUp()
        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis["uuid"])
        self.callback_url = "http://example.com/callback"
        self.agent_version = "1.0"
        self.agent_token = "test_token"

    @decorators.idempotent_id("dd2b17d9-c847-49bd-acfd-b3d63ae59f2f")
    def test_ipa_heartbeat(self):
        self._microversion_test(
            BaremetalClient.ipa_heartbeat,
            "1.22",
            lib_exc.NotFound,
            {
                "node_uuid": self.node["uuid"],
                "callback_url": self.callback_url,
                "agent_version": self.agent_version,
                "agent_token": self.agent_token,
            },
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from ironic_tempest_plugin.tests.scenario import \
    baremetal_standalone_manager as bsm

CONF = config.CONF


class BaremetalRedfishDHCPLessDeploy(bsm.BaremetalStandaloneScenarioTest):

    api_microversion = '1.59'  # Ussuri for redfish-virtual-media
    driver = 'redfish'
    deploy_interface = 'direct'
    boot_interface = 'redfish-virtual-media'
    # To force interface retoggling.
    image_ref = CONF.baremetal.whole_disk_image_ref
    image_checksum = CONF.baremetal.whole_disk_image_checksum
    wholedisk_image = True

    @classmethod
    def skip_checks(cls):
        super(BaremetalRedfishDHCPLessDeploy, cls).skip_checks()
        if CONF.baremetal_feature_enabled.dhcpless_vmedia:
            raise cls.skipException("This test requires a full OS image to "
                                    "be deployed, and thus must be "
                                    "explicitly enabled for testing.")

        if (not CONF.baremetal.public_subnet_id
                or not CONF.baremetal.public_subnet_ip):
            raise cls.skipException(
                "This test requires a public sunbet ID, and public subnet "
                "IP to use on that subnet to execute. Please see the "
                "baremetal configuration options public_subnet_id "
                "and public_subnet_ip respectively, and populate with "
                "appropriate values to execute this test.")

    def create_tenant_network(self, clients, tenant_cidr, ip_version):
        # NOTE(TheJulia): self.create_network is an internal method
        # which just gets the info, doesn't actually create a network.
        network = self.create_network(
            networks_client=self.os_admin.networks_client,
            project_id=clients.credentials.project_id,
            shared=True)

        router = self.get_router(
            client=clients.routers_client,
            project_id=clients.credentials.tenant_id,
            external_gateway_info={
                'network_id': CONF.network.public_network_id,
                'external_fixed_ips': [
                    {'subnet_id': CONF.baremetal.public_subnet_id,
                     'ip_address': CONF.baremetal.public_subnet_ip}]
            })
        result = clients.subnets_client.create_subnet(
            name=data_utils.rand_name('subnet'),
            network_id=network['id'],
            tenant_id=clients.credentials.tenant_id,
            ip_version=CONF.validation.ip_version_for_ssh,
            cidr=tenant_cidr, enable_dhcp=False)
        subnet = result['subnet']
        clients.routers_client.add_router_interface(router['id'],
                                                    subnet_id=subnet['id'])
        self.addCleanup(clients.subnets_client.delete_subnet, subnet['id'])
        self.addCleanup(clients.routers_client.remove_router_interface,
                        router['id'], subnet_id=subnet['id'])
        return network, subnet, router

    def deploy_vmedia_dhcpless(self, rebuild=False):
        """Helper to facilitate vmedia testing.

        * Create Network/router without DHCP
        * Set provisioning_network for this node.
        * Set cleanup to undo the provisionign network setup.
        * Launch instance.
          * Requirement: Instance OS image supports network config from
            network_data embedded in the OS. i.e. a real image, not
            cirros.
        * If so enabled, rebuild the node, Verify rebuild completed.
        * Via cleanup: Teardown Network/Router
        """

        # Get the latest state for the node.
        self.node = self.get_node(self.node['uuid'])
        # This test, as far as I'm remembering after the fact, was developed
        # in an environment where neutron was the default network interface.
        # so we must try to set it to properly ensure dhcp-less operation.
        prior_prov_net = self.node['driver_info'].get('provisioning_network')
        try:
            self.client.update_node(self.node['uuid'],
                                    [{'path': '/network_interface',
                                      'op': 'replace',
                                      'value': 'neutron'}])
            self.addCleanup(self.update_node,
                            self.node['uuid'],
                            [{'op': 'replace',
                              'path': '/network_interface',
                              'value': 'flat'}])
        except Exception:
            raise self.skipException(
                "Ironic configuration incorrect to exercise this test.")

        ip_version = CONF.validation.ip_version_for_ssh
        tenant_cidr = '10.0.6.0/24'
        if ip_version == 6:
            tenant_cidr = 'fd00:33::/64'

        network, subnet, router = self.create_tenant_network(
            self.os_admin, tenant_cidr, ip_version=ip_version)
        if prior_prov_net:
            self.update_node(self.node['uuid'],
                             [{'op': 'replace',
                               'path': '/driver_info/provisioning_network',
                               'value': network['id']}])
            self.addCleanup(self.update_node,
                            self.node['uuid'],
                            [{'op': 'replace',
                              'path': '/driver_info/provisioning_network',
                              'value': prior_prov_net}])
        else:
            self.update_node(self.node['uuid'],
                             [{'op': 'add',
                               'path': '/driver_info/provisioning_network',
                               'value': network['id']}])
            self.addCleanup(self.update_node,
                            self.node['uuid'],
                            [{'op': 'remove',
                              'path': '/driver_info/provisioning_network'}])

        self.set_node_to_active(self.image_ref, self.image_checksum,
                                fallback_network=network['id'],
                                config_drive_networking=True,
                                method_to_get_ip=self.get_server_ip)

        # node_ip is set by the prior call to set_node_to_active
        self.assertTrue(self.ping_ip_address(self.node_ip))

        if rebuild:
            self.set_node_provision_state(self.node['uuid'], 'rebuild')
            self.wait_provisioning_state(self.node['uuid'], 'active',
                                         timeout=CONF.baremetal.active_timeout,
                                         interval=30)
            # NOTE(TheJulia): We explicitly don't ping in this test as
            # what we are testing is that we are able to return to active.
            # i.e. That the agent is operating as designed.
        # Force delete so we remove the vifs
        self.terminate_node(self.node['uuid'], force_delete=True)

    @decorators.idempotent_id('1f420ef3-99bd-46c7-b859-ce9c2892697f')
    @utils.services('image', 'network')
    def test_ip_access_to_server(self):
        self.deploy_vmedia_dhcpless(
            rebuild=CONF.baremetal.rebuild_remote_dhcpless)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from ironic_tempest_plugin.tests.api.admin import api_microversion_fixture
from ironic_tempest_plugin.tests.api import base


class TestInspectionRulesNegative(base.BaseBaremetalTest):
    """Negative Inspection Rules test"""

    # Inspection rules API was introduced in microversion 1.96
    # We will be skipping this test for the older version.
    min_microversion = '1.96'

    def setUp(self):
        super(TestInspectionRulesNegative, self).setUp()

        _, self.node = self.create_node(None)

        self.useFixture(
            api_microversion_fixture.APIMicroversionFixture('1.96'))

    def _create_inspection_rule_payload(self, **kwargs):
        """Create a Inspection rule payload."""
        payload = {
            "description": "Inspection rule to log node UUID",
            "conditions": [],
            "actions": [
                {
                    "op": "log",
                    "args": {
                        "msg": "Node with UUID {node.uuid} is being inspected"
                    }
                }
            ],
            "phase": "main",
            "priority": 0,
            "sensitive": False
        }

        payload.update(kwargs)

        return payload

    @decorators.idempotent_id('55403d94-53ce-41ab-989a-da3399314c9d')
    @decorators.attr(type=['negative'])
    def test_create_invalid_priority_fails(self):
        """Test to create Inspection rule with invalid priorities"""
        invalid_priorities = [-1, 10000, 5000.50]

        for priority_val in invalid_priorities:
            payload = self._create_inspection_rule_payload(
                priority=priority_val)

            self.assertRaises(lib_exc.BadRequest,
                              self.create_inspection_rule,
                              rule_uuid=None, payload=payload)

    @decorators.idempotent_id('cf9615b3-904e-4456-b00a-622d39892b88')
    @decorators.attr(type=['negative'])
    def test_delete_by_wrong_uiid(self):
        """Test to delete Inspection Rule with wrong uuid"""
        rule_uuid = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.delete_inspection_rule,
                          rule_uuid=rule_uuid)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from ironic_tempest_plugin.tests.api import base


EXAMPLE_STEPS = [{
    'interface': 'bios',
    'step': 'apply_configuration',
    'args': {},
    'order': 1
}]


def _get_random_trait():
    return data_utils.rand_name('CUSTOM', '').replace('-', '_')


class TestRunbookRBAC(base.BaseBaremetalRBACTest):
    min_microversion = '1.92'
    credentials = ['system_admin',
                   'system_reader',
                   'project_admin',
                   'project_member']

    def setUp(self):
        super(TestRunbookRBAC, self).setUp()
        self.system_admin_client = (
            self.os_system_admin.baremetal.BaremetalClient())

        self.system_reader_client = (
            self.os_system_reader.baremetal.BaremetalClient())

        self.project_admin_client = (
            self.os_project_admin.baremetal.BaremetalClient())

        self.project_member_client = (
            self.os_project_member.baremetal.BaremetalClient())

        _, self.chassis = self.create_chassis()
        _, self.node = self.create_node(self.chassis['uuid'])

        _, self.public_runbook = self.system_admin_client.create_runbook(
            _get_random_trait(), steps=EXAMPLE_STEPS, public=True)

        _, self.private_runbook = self.project_admin_client.create_runbook(
            _get_random_trait(), steps=EXAMPLE_STEPS)

    @decorators.idempotent_id('0410fbed-3454-ded3-6f8b3192abfd3fcd')
    def test_runbook_visibility_and_access_control(self):
        # Project-scoped user can see both public and owned runbooks
        _, project_runbooks = self.project_member_client.list_runbooks()
        self.assertIn(self.public_runbook['uuid'],
                      [r['uuid'] for r in project_runbooks['runbooks']])
        self.assertIn(self.private_runbook['uuid'],
                      [r['uuid'] for r in project_runbooks['runbooks']])

        # System-scoped user can see all runbooks
        _, system_runbooks = self.system_reader_client.list_runbooks()
        self.assertIn(self.public_runbook['uuid'],
                      [r['uuid'] for r in system_runbooks['runbooks']])
        self.assertIn(self.private_runbook['uuid'],
                      [r['uuid'] for r in system_runbooks['runbooks']])

    @decorators.idempotent_id('903d0027-9265-9f87-c86d-09867aa24edd')
    def test_runbook_ownership_and_public_flag(self):
        # Only system-scoped users can set a runbook as public

        patch_public = [{'path': '/public', 'op': 'replace', 'value': True}]
        self.assertRaises(lib_exc.Forbidden,
                          self.project_admin_client.update_runbook,
                          self.private_runbook['uuid'],
                          patch=patch_public)

        # Setting a runbook as public nullifies its owner field
        self.system_admin_client.update_runbook(self.private_runbook['uuid'],
                                                patch=patch_public)
        _, updated_runbook = self.system_admin_client.show_runbook(
            self.private_runbook['uuid'])
        self.assertIsNone(updated_runbook['owner'])

        # Project-scoped user cannot change the owner of a runbook
        patch_owner = [{'path': '/public', 'op': 'replace', 'value': True}]
        self.assertRaises(lib_exc.Forbidden,
                          self.project_admin_client.update_runbook,
                          self.public_runbook['uuid'],
                          patch=patch_owner)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc

from ironic_tempest_plugin.common import utils
from ironic_tempest_plugin import exceptions as ironic_exc

LOG = log.getLogger(__name__)

CONF = config.CONF


def _determine_and_check_timeout_interval(timeout, default_timeout,
                                          interval, default_interval):
    if timeout is None:
        timeout = default_timeout
    if interval is None:
        interval = default_interval
    if (not isinstance(timeout, int)
            or not isinstance(interval, int)
            or timeout < 0 or interval < 0):
        raise AssertionError(
            'timeout and interval should be >= 0 or None, current values are: '
            '%(timeout)s, %(interval)s respectively. If timeout and/or '
            'interval are None, the default_timeout and default_interval are '
            'used, and they should be integers >= 0, current values are: '
            '%(default_timeout)s, %(default_interval)s respectively.' % dict(
                timeout=timeout, interval=interval,
                default_timeout=default_timeout,
                default_interval=default_interval)
        )
    return timeout, interval


def wait_for_bm_node_status(client, node_id, attr, status, timeout=None,
                            interval=None, abort_on_error_state=False):
    """Waits for a baremetal node attribute to reach given status.

    :param client: an instance of tempest plugin BaremetalClient.
    :param node_id: identifier of the node.
    :param attr: node's API-visible attribute to check status of.
    :param status: desired status. Can be a list of statuses.
    :param timeout: the timeout after which the check is considered as failed.
        Defaults to client.build_timeout.
    :param interval: an interval between show_node calls for status check.
        Defaults to client.build_interval.
    :param abort_on_error_state: whether to abort waiting if the node reaches
        an error state.

    The client should have a show_node(node_id) method to get the node.
    """
    timeout, interval = _determine_and_check_timeout_interval(
        timeout, client.build_timeout, interval, client.build_interval)

    if not isinstance(status, list):
        status = [status]

    def is_attr_in_status():
        node = utils.get_node(client, node_id=node_id)
        if node[attr] in status:
            return True
        elif (abort_on_error_state
              and (node['provision_state'].endswith(' failed')
                   or node['provision_state'] == 'error')):
            msg = ('Node %(node)s reached failure state %(state)s while '
                   'waiting for %(attr)s=%(expected)s. '
                   'Error: %(error)s' %
                   {'node': node_id, 'state': node['provision_state'],
                    'attr': attr, 'expected': status,
                    'error': node.get('last_error')})
            LOG.debug(msg)
            raise lib_exc.TempestException(msg)
        return False

    if not test_utils.call_until_true(is_attr_in_status, timeout,
                                      interval):
        message = ('Node %(node_id)s failed to reach %(attr)s=%(status)s '
                   'within the required time (%(timeout)s s).' %
                   {'node_id': node_id,
                    'attr': attr,
                    'status': status,
                    'timeout': timeout})
        caller = test_utils.find_test_caller()
        if caller:
            message = '(%s) %s' % (caller, message)
        LOG.debug(message)
        raise lib_exc.TimeoutException(message)


def wait_node_instance_association(client, instance_uuid, timeout=None,
                                   interval=None):
    """Waits for a node to be associated with instance_id.

    :param client: an instance of tempest plugin BaremetalClient.
    :param instance_uuid: UUID of the instance.
    :param timeout: the timeout after which the check is considered as failed.
        Defaults to CONF.baremetal.association_timeout.
    :param interval: an interval between show_node calls for status check.
        Defaults to client.build_interval.
    """
    timeout, interval = _determine_and_check_timeout_interval(
        timeout, CONF.baremetal.association_timeout,
        interval, client.build_interval)

    def is_some_node_associated():
        node = utils.get_node(client, instance_uuid=instance_uuid)
        return node is not None

    if not test_utils.call_until_true(is_some_node_associated, timeout,
                                      interval):
        msg = ('Timed out waiting to get Ironic node by instance UUID '
               '%(instance_uuid)s within the required time (%(timeout)s s).'
               % {'instance_uuid': instance_uuid, 'timeout': timeout})
        raise lib_exc.TimeoutException(msg)


def wait_for_allocation(client, allocation_ident, timeout=15, interval=1,
                        expect_error=False):
    """Wait for the allocation to become active.

    :param client: an instance of tempest plugin BaremetalClient.
    :param allocation_ident: UUID or name of the allocation.
    :param timeout: the timeout after which the allocation is considered as
        failed. Defaults to 15 seconds.
    :param interval: an interval between show_allocation calls.
        Defaults to 1 second.
    :param expect_error: if True, return successfully even in case of an error.
    """
    result = [None]  # a mutable object to modify in the closure

    def check():
        result[0] = client.show_allocation(allocation_ident)
        allocation = result[0][1]

        if allocation['state'] == 'error' and not expect_error:
            raise lib_exc.TempestException(
                "Allocation %(ident)s failed: %(error)s" %
                {'ident': allocation_ident,
                 'error': allocation.get('last_error')})
        else:
            return allocation['state'] != 'allocating'

    if not test_utils.call_until_true(check, timeout, interval):
        msg = ('Timed out waiting for the allocation %s to become active' %
               allocation_ident)
        raise lib_exc.TimeoutException(msg)

    return result[0]


def wait_node_value_in_field(client, node_id, field, value,
                             raise_if_insufficent_access=True,
                             timeout=None, interval=None,
                             abort_on_error_state=False):
    """Waits for a node to have a field value appear.

    :param client: an instance of tempest plugin BaremetalClient.
    :param node_id: the UUID of the node
    :param field: the field in the node object to examine
    :param value: the value/key with-in the field to look for.
    :param timeout: the timeout after which the check is considered as failed.
    :param interval: an interval between show_node calls for status check.
    :param abort_on_error_state: whether to abort waiting if the node reaches
        an error state.
    """

    def is_field_updated():
        node = utils.get_node(client, node_id=node_id)
        field_value = node[field]
        if raise_if_insufficent_access and '** Redacted' in field_value:
            msg = ('Unable to see contents of redacted field '
                   'indicating insufficient access to execute this test.')
            raise ironic_exc.InsufficientAPIAccess(msg)
        elif (abort_on_error_state
              and (node['provision_state'].endswith('failed')
                   or node['provision_state'] == 'error')):
            msg = ('Node %(node)s reached failure state %(state)s while '
                   'waiting Error: %(error)s' %
                   {'node': node_id, 'state': node['provision_state'],
                    'error': node.get('last_error')})
            LOG.debug(msg)
            raise lib_exc.TempestException(msg)
        return value in field_value

    if not test_utils.call_until_true(is_field_updated, timeout,
                                      interval):
        msg = ('Timed out waiting to get Ironic node by node_id '
               '%(node_id)s within the required time (%(timeout)s s). '
               'Field value %(value) did not appear in field %(field)s.'
               % {'node_id': node_id, 'timeout': timeout,
                  'field': field, 'value': value})
        raise lib_exc.TimeoutException(msg)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.config import CONF
from tempest.lib import decorators

from ironic_tempest_plugin.tests.scenario import baremetal_manager
from ironic_tempest_plugin.tests.scenario import introspection_manager


class InspectorBasicTest(introspection_manager.InspectorScenarioTest):

    def verify_node_introspection_data(self, node):
        data = self.introspection_data(node['uuid'])
        self.assertEqual(data['cpu_arch'],
                         self.flavor['properties']['cpu_arch'])
        self.assertGreater(int(data['memory_mb']), 0)
        self.assertGreater(int(data['cpus']), 0)

    def verify_node_flavor(self, node):
        expected_cpu_arch = self.flavor['properties']['cpu_arch']

        self.assertGreater(int(node['properties']['cpus']), 0)
        self.assertGreater(int(node['properties']['memory_mb']), 0)
        self.assertGreater(int(node['properties']['local_gb']), 0)
        self.assertEqual(expected_cpu_arch,
                         node['properties']['cpu_arch'])

    def verify_introspection_aborted(self, uuid):
        status = self.introspection_status(uuid)

        self.assertEqual('Canceled by operator', status['error'])
        self.assertTrue(status['finished'])

        self.wait_provisioning_state(
            uuid, 'inspect failed',
            timeout=CONF.baremetal.active_timeout,
            interval=self.wait_provisioning_state_interval)

    @decorators.idempotent_id('03bf7990-bee0-4dd7-bf74-b97ad7b52a4b')
    @decorators.attr(type='smoke')
    @utils.services('compute', 'image', 'network')
    def test_baremetal_introspection(self):
        """This smoke test case follows this set of operations:

            * Fetches expected properties from baremetal flavor
            * Removes all properties from nodes
            * Sets nodes to manageable state
            * Imports introspection rule basic_ops_rule.json
            * Inspects nodes
            * Verifies all properties are inspected
            * Verifies introspection data
            * Sets node to available state

        """
        # prepare introspection rule
        rule_path = self.get_rule_path("basic_ops_rule.json")
        self.rule_import(rule_path)
        self.addCleanup(self.rule_purge)

        for node_id in self.node_ids:
            self.introspect_node(node_id)

        # settle down introspection
        self.wait_for_introspection_finished(self.node_ids)
        for node_id in self.node_ids:
            self.wait_provisioning_state(
                node_id, 'manageable',
                timeout=CONF.baremetal_introspection.ironic_sync_timeout,
                interval=self.wait_provisioning_state_interval)

        for node_id in self.node_ids:
            node = self.node_show(node_id)
            self.assertEqual('yes', node['extra']['rule_success'])
            data_store = CONF.baremetal_introspection.data_store
            if data_store is None:
                # Backward compatibility, the option is not set.
                data_store = ('swift' if CONF.service_available.swift
                              else 'none')
            if data_store != 'none':
                self.verify_node_introspection_data(node)
            self.verify_node_flavor(node)

        for node_id in self.node_ids:
            self.baremetal_client.set_node_provision_state(node_id, 'provide')

        for node_id in self.node_ids:
            self.wait_provisioning_state(
                node_id, baremetal_manager.BaremetalProvisionStates.AVAILABLE,
                timeout=CONF.baremetal.active_timeout,
                interval=self.wait_provisioning_state_interval)

    @decorators.idempotent_id('70ca3070-184b-4b7d-8892-e977d2bc2870')
    def test_introspection_abort(self):
        """This smoke test case follows this very basic set of operations:

            * Start nodes introspection
            * Wait until nodes power on
            * Abort introspection
            * Verifies nodes status and power state

        """
        # start nodes introspection
        for node_id in self.node_ids:
            self.introspect_node(node_id, remove_props=False)

        # wait for nodes power on
        for node_id in self.node_ids:
            self.wait_power_state(
                node_id,
                baremetal_manager.BaremetalPowerStates.POWER_ON)

        # abort introspection
        for node_id in self.node_ids:
            self.introspection_abort(node_id)

        # wait for nodes power off
        for node_id in self.node_ids:
            self.wait_power_state(
                node_id,
                baremetal_manager.BaremetalPowerStates.POWER_OFF)

        # verify nodes status and provision state
        for node_id in self.node_ids:
            self.verify_introspection_aborted(node_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools

from oslo_log import log as logging
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as lib_exc
from tempest import test

from ironic_tempest_plugin.common import waiters
from ironic_tempest_plugin.services.baremetal import base
from ironic_tempest_plugin.tests.api.admin import api_microversion_fixture


LOG = logging.getLogger(__name__)
CONF = config.CONF


# NOTE(adam_g): The baremetal API tests exercise operations such as enroll
# node, power on, power off, etc.  Testing against real drivers (ie, IPMI)
# will require passing driver-specific data to Tempest (addresses,
# credentials, etc).  Until then, only support testing against the fake driver,
# which has no external dependencies.
SUPPORTED_DRIVERS = ['fake', 'fake-hardware']

# NOTE(jroll): resources must be deleted in a specific order, this list
# defines the resource types to clean up, and the correct order.
RESOURCE_TYPES = ['port', 'portgroup', 'node', 'volume_connector',
                  'volume_target', 'chassis', 'deploy_template',
                  'runbook', 'inspection_rule']


def creates(resource):
    """Decorator that adds resources to the appropriate cleanup list."""

    def decorator(f):
        @functools.wraps(f)
        def wrapper(cls, *args, **kwargs):
            resp, body = f(cls, *args, **kwargs)

            if 'uuid' in body:
                cls.created_objects[resource].add(body['uuid'])

            return resp, body
        return wrapper
    return decorator


class BaseBaremetalTest(api_version_utils.BaseMicroversionTest,
                        test.BaseTestCase):
    """Base class for Baremetal API tests."""

    credentials = ['admin', 'system_admin']

    @classmethod
    def skip_checks(cls):
        super(BaseBaremetalTest, cls).skip_checks()
        if not CONF.service_available.ironic:
            raise cls.skipException('Ironic is not enabled.')
        if CONF.baremetal.driver not in SUPPORTED_DRIVERS:
            skip_msg = ('%s skipped as Ironic driver %s is not supported for '
                        'testing.' %
                        (cls.__name__, CONF.baremetal.driver))
            raise cls.skipException(skip_msg)

        cfg_min_version = CONF.baremetal.min_microversion
        cfg_max_version = CONF.baremetal.max_microversion

        # Check versions and skip based upon *configuration*
        api_version_utils.check_skip_with_microversion(cls.min_microversion,
                                                       cls.max_microversion,
                                                       cfg_min_version,
                                                       cfg_max_version)

    @classmethod
    def setup_credentials(cls):
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.baremetal.min_microversion))
        cls.services_microversion = {
            CONF.baremetal.catalog_type: cls.request_microversion}

        super(BaseBaremetalTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BaseBaremetalTest, cls).setup_clients()
        if CONF.enforce_scope.ironic:
            cls.client = cls.os_system_admin.baremetal.BaremetalClient()
        else:
            cls.client = cls.os_admin.baremetal.BaremetalClient()

        # Skip the test if the class version doesn't match.
        if cls.min_microversion or cls.max_microversion:
            api_min, api_max = cls.client.get_min_max_api_microversions()
            api_version_utils.check_skip_with_microversion(
                cls.min_microversion,
                cls.max_microversion,
                api_min,
                api_max)

    @classmethod
    def resource_setup(cls):
        super(BaseBaremetalTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.baremetal.min_microversion))
        cls.driver = CONF.baremetal.driver
        cls.power_timeout = CONF.baremetal.power_timeout
        cls.unprovision_timeout = CONF.baremetal.unprovision_timeout
        cls.created_objects = {}
        for resource in RESOURCE_TYPES + ['allocation']:
            cls.created_objects[resource] = set()
        cls.deployed_nodes = set()

    @classmethod
    def resource_cleanup(cls):
        """Ensure that all created objects get destroyed."""
        # Use the requested microversion for cleanup to ensure we can delete
        # resources.
        base.set_baremetal_api_microversion(cls.request_microversion)
        try:
            for node in cls.deployed_nodes:
                try:
                    cls.set_node_provision_state(node, 'deleted',
                                                 ['available', None])
                except lib_exc.BadRequest:
                    LOG.warning('Cleanup: Failed to unprovision node: %s',
                                node)

            # Delete allocations explicitly after unprovisioning instances, but
            # before deleting nodes.
            for allocation in cls.created_objects['allocation']:
                try:
                    cls.client.delete_allocation(allocation)
                except lib_exc.NotFound:
                    LOG.warning('Cleanup: Failed to delete allocation: %s',
                                allocation)

            for node in cls.created_objects['node']:
                try:
                    cls.client.update_node(node, instance_uuid=None)
                except lib_exc.TempestException:
                    LOG.warning('Cleanup: Failed to delete node: %s',
                                node)

            for resource in RESOURCE_TYPES:
                uuids = cls.created_objects[resource]
                delete_method = getattr(cls.client, 'delete_%s' % resource)
                for u in uuids:
                    delete_method(u, ignore_errors=lib_exc.NotFound)
        finally:
            base.reset_baremetal_api_microversion()
            super(BaseBaremetalTest, cls).resource_cleanup()

    def _assertExpected(self, expected, actual):
        """Check if expected keys/values exist in actual response body.

        Check if the expected keys and values are in the actual response body.
        It will not check the keys 'created_at' and 'updated_at', since they
        will always have different values. Asserts if any expected key (or
        corresponding value) is not in the actual response.

        Note: this method has an underscore even though it is used outside of
        this class, in order to distinguish this method from the more standard
        assertXYZ methods.

        :param expected: dict of key-value pairs that are expected to be in
                         'actual' dict.
        :param actual: dict of key-value pairs.

        """
        for key, value in expected.items():
            if key not in ('created_at', 'updated_at'):
                self.assertIn(key, actual)
                self.assertEqual(value, actual[key])

    def setUp(self):
        super(BaseBaremetalTest, self).setUp()
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            self.request_microversion))

    @classmethod
    @creates('chassis')
    def create_chassis(cls, description=None, **kwargs):
        """Wrapper utility for creating test chassis.

        :param description: A description of the chassis. If not supplied,
            a random value will be generated.
        :return: A tuple with the server response and the created chassis.

        """
        description = description or data_utils.rand_name('test-chassis')
        resp, body = cls.client.create_chassis(description=description,
                                               **kwargs)
        return resp, body

    @classmethod
    @creates('node')
    def create_node(cls, chassis_id, cpu_arch='x86_64', cpus=8, local_gb=10,
                    memory_mb=4096, **kwargs):
        """Wrapper utility for creating test baremetal nodes.

        :param chassis_id: The unique identifier of the chassis.
        :param cpu_arch: CPU architecture of the node. Default: x86_64.
        :param cpus: Number of CPUs. Default: 8.
        :param local_gb: Disk size. Default: 10.
        :param memory_mb: Available RAM. Default: 4096.
        :param kwargs: Other optional node fields.
        :return: A tuple with the server response and the created node.

        """
        resp, body = cls.client.create_node(chassis_id, cpu_arch=cpu_arch,
                                            cpus=cpus, local_gb=local_gb,
                                            memory_mb=memory_mb,
                                            driver=cls.driver,
                                            **kwargs)

        return resp, body

    @classmethod
    def set_node_provision_state(cls, node_id, target, expected, timeout=None,
                                 interval=None):
        """Sets the node's provision state.

        :param node_id: The unique identifier of the node.
        :param target: Target provision state.
        :param expected: Expected final provision state or list of states.
        :param timeout: The timeout for reaching the expected state.
            Defaults to client.build_timeout.
        :param interval: An interval between show_node calls for status check.
            Defaults to client.build_interval.
        """
        cls.client.set_node_provision_state(node_id, target)
        waiters.wait_for_bm_node_status(cls.client, node_id,
                                        'provision_state', expected,
                                        timeout=timeout, interval=interval)

    @classmethod
    def provide_node(cls, node_id, cleaning_timeout=None):
        """Make the node available.

        :param node_id: The unique identifier of the node.
        :param cleaning_timeout: The timeout to wait for cleaning.
            Defaults to client.build_timeout.
        """
        _, body = cls.client.show_node(node_id)
        current_state = body['provision_state']
        if current_state == 'enroll':
            cls.set_node_provision_state(node_id, 'manage', 'manageable',
                                         timeout=60, interval=1)
            current_state = 'manageable'
        if current_state == 'manageable':
            cls.set_node_provision_state(node_id, 'provide',
                                         ['available', None],
                                         timeout=cleaning_timeout)
            current_state = 'available'
        if current_state not in ('available', None):
            raise RuntimeError("Cannot reach state 'available': node %(node)s "
                               "is in unexpected state %(state)s" %
                               {'node': node_id, 'state': current_state})

    @classmethod
    def deploy_node(cls, node_id, cleaning_timeout=None, deploy_timeout=None):
        """Deploy the node.

        :param node_id: The unique identifier of the node.
        :param cleaning_timeout: The timeout to wait for cleaning.
            Defaults to client.build_timeout.
        :param deploy_timeout: The timeout to wait for deploy.
            Defaults to client.build_timeout.
        """
        cls.provide_node(node_id, cleaning_timeout=cleaning_timeout)
        cls.set_node_provision_state(node_id, 'active', 'active',
                                     timeout=deploy_timeout)
        cls.deployed_nodes.add(node_id)

    @classmethod
    @creates('port')
    def create_port(cls, node_id, address, extra=None, uuid=None,
                    portgroup_uuid=None, physical_network=None):
        """Wrapper utility for creating test ports.

        :param node_id: The unique identifier of the node.
        :param address: MAC address of the port.
        :param extra: Meta data of the port. If not supplied, an empty
            dictionary will be created.
        :param uuid: UUID of the port.
        :param portgroup_uuid: The UUID of a portgroup of which this port is a
            member.
        :param physical_network: The physical network to which the port is
            attached.
        :return: A tuple with the server response and the created port.

        """
        extra = extra or {}
        resp, body = cls.client.create_port(address=address, node_id=node_id,
                                            extra=extra, uuid=uuid,
                                            portgroup_uuid=portgroup_uuid,
                                            physical_network=physical_network)

        return resp, body

    @classmethod
    @creates('portgroup')
    def create_portgroup(cls, node_uuid, **kwargs):
        """Wrapper utility for creating test port groups.

        :param node_uuid: The unique identifier of the node.
        :return: A tuple with the server response and the created port group.
        """
        resp, body = cls.client.create_portgroup(node_uuid=node_uuid, **kwargs)

        return resp, body

    @classmethod
    @creates('volume_connector')
    def create_volume_connector(cls, node_uuid, **kwargs):
        """Wrapper utility for creating test volume connector.

        :param node_uuid: The unique identifier of the node.
        :return: A tuple with the server response and the created volume
            connector.
        """
        resp, body = cls.client.create_volume_connector(node_uuid=node_uuid,
                                                        **kwargs)

        return resp, body

    @classmethod
    @creates('volume_target')
    def create_volume_target(cls, node_uuid, **kwargs):
        """Wrapper utility for creating test volume target.

        :param node_uuid: The unique identifier of the node.
        :return: A tuple with the server response and the created volume
            target.
        """
        resp, body = cls.client.create_volume_target(node_uuid=node_uuid,
                                                     **kwargs)

        return resp, body

    @classmethod
    @creates('deploy_template')
    def create_deploy_template(cls, name, **kwargs):
        """Wrapper utility for creating test deploy template.

        :param name: The name of the deploy template.
        :return: A tuple with the server response and the created deploy
            template.
        """
        resp, body = cls.client.create_deploy_template(name=name, **kwargs)

        return resp, body

    @classmethod
    @creates('runbook')
    def create_runbook(cls, name, **kwargs):
        """Wrapper utility for creating test runbook.

        :param name: The name of the runbook.
        :return: A tuple with the server response and the created runbook.
        """
        resp, body = cls.client.create_runbook(name=name, **kwargs)

        return resp, body

    @classmethod
    def delete_chassis(cls, chassis_id):
        """Deletes a chassis having the specified UUID.

        :param chassis_id: The unique identifier of the chassis.
        :return: Server response.

        """

        resp, body = cls.client.delete_chassis(chassis_id)

        if chassis_id in cls.created_objects['chassis']:
            cls.created_objects['chassis'].remove(chassis_id)

        return resp

    @classmethod
    def delete_node(cls, node_id):
        """Deletes a node having the specified UUID.

        :param node_id: The unique identifier of the node.
        :return: Server response.

        """

        resp, body = cls.client.delete_node(node_id)

        if node_id in cls.created_objects['node']:
            cls.created_objects['node'].remove(node_id)

        return resp

    @classmethod
    def delete_port(cls, port_id):
        """Deletes a port having the specified UUID.

        :param port_id: The unique identifier of the port.
        :return: Server response.

        """

        resp, body = cls.client.delete_port(port_id)

        if port_id in cls.created_objects['port']:
            cls.created_objects['port'].remove(port_id)

        return resp

    @classmethod
    def delete_portgroup(cls, portgroup_ident):
        """Deletes a port group having the specified UUID or name.

        :param portgroup_ident: The name or UUID of the port group.
        :return: Server response.
        """
        resp, body = cls.client.delete_portgroup(portgroup_ident)

        if portgroup_ident in cls.created_objects['portgroup']:
            cls.created_objects['portgroup'].remove(portgroup_ident)

        return resp

    @classmethod
    def delete_volume_connector(cls, volume_connector_id):
        """Deletes a volume connector having the specified UUID.

        :param volume_connector_id: The UUID of the volume connector.
        :return: Server response.
        """
        resp, body = cls.client.delete_volume_connector(volume_connector_id)

        if volume_connector_id in cls.created_objects['volume_connector']:
            cls.created_objects['volume_connector'].remove(
                volume_connector_id)

        return resp

    @classmethod
    def delete_volume_target(cls, volume_target_id):
        """Deletes a volume target having the specified UUID.

        :param volume_target_id: The UUID of the volume target.
        :return: Server response.
        """
        resp, body = cls.client.delete_volume_target(volume_target_id)

        if volume_target_id in cls.created_objects['volume_target']:
            cls.created_objects['volume_target'].remove(volume_target_id)

        return resp

    @classmethod
    def delete_deploy_template(cls, deploy_template_ident):
        """Deletes a deploy template having the specified name or UUID.

        :param deploy_template_ident: Name or UUID of the deploy template.
        :return: Server response.
        """
        resp, body = cls.client.delete_deploy_template(deploy_template_ident)

        if deploy_template_ident in cls.created_objects['deploy_template']:
            cls.created_objects['deploy_template'].remove(
                deploy_template_ident)

        return resp

    @classmethod
    def delete_runbook(cls, runbook_ident):
        """Deletes a runbook having the specified name or UUID.

        :param runbook_ident: Name or UUID of the runbook.
        :return: Server response.
        """
        resp, body = cls.client.delete_runbook(runbook_ident)

        if runbook_ident in cls.created_objects['runbook']:
            cls.created_objects['runbook'].remove(runbook_ident)

        return resp

    def validate_self_link(self, resource, uuid, link):
        """Check whether the given self link formatted correctly."""
        expected_link = "{base}/{pref}/{res}/{uuid}".format(
                        base=self.client.base_url.rstrip('/'),
                        pref=self.client.uri_prefix,
                        res=resource,
                        uuid=uuid)
        self.assertEqual(expected_link, link)

    @classmethod
    @creates('allocation')
    def create_allocation(cls, resource_class, **kwargs):
        """Wrapper utility for creating test allocations.

        :param resource_class: Resource class to request.
        :param kwargs: Other fields to pass.
        :return: A tuple with the server response and the created allocation.
        """
        resp, body = cls.client.create_allocation(resource_class, **kwargs)
        return resp, body

    @classmethod
    @creates('inspection_rule')
    def create_inspection_rule(cls, rule_uuid, payload):
        """Wrapper utility for creating Inspection rule.

        :param rule_uuid: UUID of the Inspection rule.
        :param payload: Inspection rule other fields.
        :return: Server response.
        """
        if rule_uuid is not None:
            payload['uuid'] = rule_uuid

        resp, body = cls.client.create_inspection_rule(payload)

        return resp, body

    @classmethod
    def delete_inspection_rule(cls, rule_uuid):
        """Delete a inspection rules having the specified UUID.

        :param rule_uuid: UUID of the Inspection rule.
        """
        resp, body = cls.client.delete_inspection_rule(rule_uuid)

        return resp


class BaseBaremetalRBACTest(BaseBaremetalTest):

    # Unless otherwise superseded by a version, RBAC tests generally start at
    # version 1.70 as that is when System scope and the delineation occurred.
    min_microversion = '1.70'

    @classmethod
    def skip_checks(cls):
        super(BaseBaremetalRBACTest, cls).skip_checks()
        if not CONF.enforce_scope.ironic:
            raise cls.skipException('RBAC tests for Ironic are not enabled.')



#
# Copyright 2018 Red Hat Inc.
#
# Copyright (c) 2022 Dell Inc. or its subsidiaries.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators

from ironic_tempest_plugin.tests.scenario import \
    baremetal_standalone_manager as bsm

LOG = logging.getLogger(__name__)
CONF = config.CONF


class BaremetalFakeBios(
        bsm.BaremetalStandaloneScenarioTest):

    driver = 'fake-hardware'
    bios_interface = 'fake'
    deploy_interface = 'iscsi'
    image_ref = CONF.baremetal.whole_disk_image_ref
    wholedisk_image = True
    delete_node = False
    api_microversion = '1.40'

    @decorators.idempotent_id('ef55c44a-cc10-4cf6-8fda-85f0c0793150')
    def test_bios_apply_and_reset_configuration(self):
        settings = [
            {
                "name": "setting1_name",
                "value": "setting1_value"
            },
            {
                "name": "setting2_name",
                "value": "setting2_value"
            }
        ]

        self.check_bios_apply_and_reset_configuration(self.node, settings)


class BaremetalIdracBiosCleaning(
        bsm.BaremetalStandaloneScenarioTest):

    mandatory_attr = ['driver', 'bios_interface']

    credentials = ['primary', 'admin']
    driver = 'idrac'
    delete_node = False
    api_microversion = '1.40'

    def _get_bios_setting_to_update(self):
        _, bios_settings = self.baremetal_client.\
            list_node_bios_settings(self.node['uuid'])

        for attr_name in bios_settings['bios']:
            if attr_name['name'] == "ProcVirtualization":
                if attr_name['value'] == 'Disabled':
                    new_attr_value = "Enabled"
                else:
                    new_attr_value = "Disabled"
        setting_to_update = [
            {
                "name": "ProcVirtualization",
                "value": new_attr_value
            }
        ]

        return setting_to_update

    def _verify_bios_settings(self, bios_settings_to_verify):
        _, current_bios_settings = self.baremetal_client.\
            list_node_bios_settings(self.node['uuid'])

        for setting in bios_settings_to_verify:
            found_setting = None
            for i in current_bios_settings['bios']:
                if i['name'] == setting['name']:
                    found_setting = i
                    break
            self.assertIsNotNone(found_setting)
            self.assertEqual(setting['value'], found_setting['value'])

    @decorators.idempotent_id('6ded82ab-b444-436b-bb78-06fa5957d6c3')
    def test_bios_apply_configuration(self):
        setting = self._get_bios_setting_to_update()
        clean_steps = [
            {
                "interface": "bios",
                "step": "apply_configuration",
                "args": {"settings": setting}
            }
        ]

        self.manual_cleaning(self.node, clean_steps=clean_steps)
        self._verify_bios_settings(setting)

        restore_setting = self._get_bios_setting_to_update()
        clean_steps_restore = [
            {
                "interface": "bios",
                "step": "apply_configuration",
                "args": {"settings": restore_setting}
            }
        ]

        self.manual_cleaning(self.node, clean_steps=clean_steps_restore)
        self._verify_bios_settings(restore_setting)


class BaremetalIdracRedfishBios(
        BaremetalIdracBiosCleaning):
    bios_interface = 'idrac-redfish'


class BaremetalIdracWSManBios(
        BaremetalIdracBiosCleaning):
    bios_interface = 'idrac-wsman'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators

from ironic_tempest_plugin.tests.api import base

CONF = config.CONF


class TestConductors(base.BaseBaremetalTest):
    """Tests for conductors."""

    min_microversion = '1.49'

    @decorators.idempotent_id('b6d62be4-53a6-43c6-ae78-bff9d1b4efc1')
    def test_list_conductors(self):
        _, conductors = self.client.list_conductors()
        self.assertTrue(len(conductors['conductors']) > 0)
        cond = conductors['conductors'].pop()
        self.assertIn('hostname', cond)
        self.assertIn('conductor_group', cond)
        self.assertIn('alive', cond)
        self.assertNotIn('drivers', cond)

    @decorators.idempotent_id('ca3de366-d80a-4e97-b19b-42d594e8d148')
    def test_list_conductors_detail(self):
        _, conductors = self.client.list_conductors(detail=True)
        self.assertTrue(len(conductors['conductors']) > 0)
        cond = conductors['conductors'].pop()
        self.assertIn('hostname', cond)
        self.assertIn('conductor_group', cond)
        self.assertIn('alive', cond)
        self.assertIn('drivers', cond)

    @decorators.idempotent_id('7e1829e2-3945-4508-a3d9-c8ebe9463fd8')
    def test_show_conductor(self):
        _, conductors = self.client.list_conductors()
        self.assertTrue(len(conductors['conductors']) > 0)
        conductor = conductors['conductors'].pop()

        _, conductor = self.client.show_conductor(conductor['hostname'])
        self.assertIn('hostname', conductor)
        self.assertIn('conductor_group', conductor)
        self.assertIn('alive', conductor)
        self.assertIn('drivers', conductor)



#
# Copyright 2017 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from tempest.common import utils
from tempest import config
from tempest.lib import decorators

from ironic_tempest_plugin.tests.scenario import \
    baremetal_standalone_manager as bsm

LOG = logging.getLogger(__name__)
CONF = config.CONF


class BaremetalAdoptionDriverWholedisk(
        bsm.BaremetalStandaloneScenarioTest):

    if 'redfish' in CONF.baremetal.enabled_hardware_types:
        driver = 'redfish'
    else:
        driver = 'ipmi'
    image_ref = CONF.baremetal.whole_disk_image_ref
    wholedisk_image = True
    deploy_interface = 'direct'
    # 1.37 is required to be able to copy traits
    api_microversion = '1.37'

    @classmethod
    def skip_checks(cls):
        super(BaremetalAdoptionDriverWholedisk, cls).skip_checks()
        if cls.driver == 'ipmi':
            skip_msg = ("Adoption feature is covered by redfish driver")
            raise cls.skipException(skip_msg)
        if not CONF.baremetal_feature_enabled.adoption:
            skip_msg = ("Adoption feature is not enabled")
            raise cls.skipException(skip_msg)

    @classmethod
    def recreate_node(cls):
        # Now record all up-to-date node information for creation
        cls.node = cls.get_node(cls.node['uuid'])
        body = {'driver_info': cls.node['driver_info'],
                'instance_info': cls.node['instance_info'],
                'driver': cls.node['driver'],
                'properties': cls.node['properties']}
        if set(body['driver_info'].get('redfish_password')) == {'*'}:
            # A hack to enable devstack testing without showing secrets
            # secrets. Use the hardcoded devstack value.
            body['driver_info']['redfish_password'] = 'password'
        # configdrive is hidden and anyway should be supplied on rebuild
        body['instance_info'].pop('configdrive', None)
        for key, value in cls.node.items():
            if key.endswith('_interface') and value:
                body[key] = value
        traits = cls.node['traits']
        _, vifs = cls.baremetal_client.vif_list(cls.node['uuid'])
        _, ports = cls.baremetal_client.list_ports(node=cls.node['uuid'])

        # Delete the active node using maintenance
        cls.update_node(cls.node['uuid'], [{'op': 'replace',
                                            'path': '/maintenance',
                                            'value': True}])
        cls.baremetal_client.delete_node(cls.node['uuid'])

        # Now create an identical node and attach VIFs
        _, cls.node = cls.baremetal_client.create_node_raw(**body)
        if traits:
            cls.baremetal_client.set_node_traits(cls.node['uuid'], traits)
        for port in ports['ports']:
            cls.baremetal_client.create_port(cls.node['uuid'],
                                             address=port['address'])

        cls.set_node_provision_state(cls.node['uuid'], 'manage')
        cls.wait_provisioning_state(cls.node['uuid'], 'manageable',
                                    timeout=300, interval=5)

        for vif in vifs['vifs']:
            cls.vif_attach(cls.node['uuid'], vif['id'])

        return cls.node

    @decorators.idempotent_id('2f51890e-20d9-43ef-af39-41b335ec066b')
    @utils.services('image', 'network')
    def test_adoption(self):
        # First, prepare a deployed node.
        self.boot_node()

        # Then re-create it with the same parameters.
        self.recreate_node()

        # Now adoption!
        self.set_node_provision_state(self.node['uuid'], 'adopt')
        self.wait_provisioning_state(self.node['uuid'], 'active',
                                     timeout=300, interval=5)

        # Try to rebuild the server to make sure we can manage it now.
        self.set_node_provision_state(self.node['uuid'], 'rebuild')
        self.wait_provisioning_state(self.node['uuid'], 'active',
                                     timeout=CONF.baremetal.active_timeout,
                                     interval=30)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from tempest.common import utils
from tempest import config
from tempest.lib import decorators

from ironic_tempest_plugin.tests.scenario import \
    baremetal_standalone_manager as bsm

LOG = logging.getLogger(__name__)
CONF = config.CONF


class BaremetalRamdiskBootIsoIPMIIPXE(bsm.BaremetalStandaloneScenarioTest):

    driver = 'ipmi'
    boot_interface = 'ipxe'
    delete_node = False
    deploy_interface = 'ramdisk'
    api_microversion = '1.66'
    image_ref = CONF.baremetal.ramdisk_iso_image_ref
    wholedisk_image = False

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not cls.image_ref:
            raise cls.skipException('Skipping ramdisk ISO booting as'
                                    'no ramdisk_iso_image_ref is defined.')

    @decorators.idempotent_id('2859d115-9266-4461-9286-79b146e65dc9')
    @utils.services('image', 'network')
    def test_ramdisk_boot(self):
        self.boot_and_verify_ramdisk_node(self.image_ref, iso=True)


class BaremetalRamdiskBootIsoRedfishVMedia(
        bsm.BaremetalStandaloneScenarioTest):

    driver = 'redfish'
    boot_interface = 'redfish-virtual-media'
    delete_node = False
    deploy_interface = 'ramdisk'
    api_microversion = '1.66'
    image_ref = CONF.baremetal.ramdisk_iso_image_ref
    wholedisk_image = False

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not cls.image_ref:
            raise cls.skipException('Skipping ramdisk ISO booting as'
                                    'no ramdisk_iso_image_ref is defined.')

    @decorators.idempotent_id('2859d115-9266-4461-9286-79b146e65dc9')
    @utils.services('image', 'network')
    def test_ramdisk_boot(self):
        self.boot_and_verify_ramdisk_node(self.image_ref, iso=True)


class BaremetalRamdiskBootIsoSNMPIPXE(bsm.BaremetalStandaloneScenarioTest):

    driver = 'snmp'
    boot_interface = 'ipxe'
    delete_node = False
    deploy_interface = 'ramdisk'
    api_microversion = '1.66'
    image_ref = CONF.baremetal.ramdisk_iso_image_ref
    wholedisk_image = False

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not cls.image_ref:
            raise cls.skipException('Skipping ramdisk ISO booting as'
                                    'no ramdisk_iso_image_ref is defined.')

    @decorators.idempotent_id('2859d115-9266-4461-9286-79b146e65dc9')
    @utils.services('image', 'network')
    def test_ramdisk_boot(self):
        self.boot_and_verify_ramdisk_node(self.image_ref, iso=True)
